/*--------------------------------------------------------------------
 *    The GMTPLUS-system:	%W%	%G%
 *
 *  Based on...
 *    The GMT-system:	@(#)binlegs.c	2.3  7/30/92
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * BINLEGS - A Program that will read a list of *.bix
 * files and update the legindex files. The bix file
 * has the index numbers of all the 1 by 1 degree boxes crossed
 * by this leg. This information is added to the legindex
 * database. This database is then used by program GMTLEGS that
 * will do the job of locating all legs within a given area (same
 * as Brown-book LOC, but now all the legs listed are really
 * inside the area).  The list of legs given to binlegs will have
 * the .bix suffix appended before opening the index file
 * 
 * Author:	Paul Wessel
 * Date:	21-MAY-1987
 * Revised:	13-SEP-1989	Added option to remove legs and specify path
 * 		2-JAN-1991	Replaced fscanf with fgets
 * 		13-FEB-1991	Allow for legnames up to 16 chars
 * Version:	2.0 21-Jun-1991
 * 
 *
 * GMTPlus version 03/10/95 jcm
 * Modifications made for gmtplus database system.
 * Since each leg may have a variable number of valid data columns,
 * the flags are now in a linked list (bin_flag_ptr) rather than bits.
 * Additionally, the max number of legs is limited by an integer leg_id
 * rather than having to fit both it and flags in an int.
 *
 */
 
#include <stdlib.h>

#include <sys/types.h> /* chmod() */
#include <sys/stat.h> /* chmod() */

#include <gmt.h>

#include "gmtplus_mgg.h"

#define S_RDONLY 0000444
 
struct GRID {
	int bix;
	int n_legs;
	struct LEG *first_leg, *last_leg;
} grid[64800];
	
struct LEG {
	int leg_id;	
	bin_flag_ptr flags;
	struct LEG *next_leg;
};
	

/* FIX: BAD PROTOTYPES - make this a real header -kds */
struct INFO *make_info();
struct LEG *make_leg();
int remove_leg();

int
main (int argc, char *argv[])
{
	struct INFO *info_head, *this_info, *new_info;
	struct LEG *this_leg;
	
	char file[80], leg[16], lname[16], line[512];
	char leg_file[120], index_file[120], old_leg_file[120], old_index_file[120], path[80];
	char error = FALSE, replace = FALSE, delete = FALSE, found_it, verbose = FALSE;
	char *envdir;
	
	FILE *fp = NULL, *fbin, *fleg, *find;
	
	int no_of_legs, index, bin, free_id, num_max, max[14], flag[14];
	int id = 1;
	int num;
	int i, j, year, last_id;
	bin_flag_ptr ptr;
    int path_specified = 0;
	
	if (argc == 1) error = TRUE;
	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			for (j = 1; argv[i][j]; j++) {
				switch (argv[i][1]) {
					case 'R':	/* Replace existing info for the legs, then append new info */
						replace = TRUE;
						break;
					case 'D':	/* Remove all traces of these legs from the info files */
						delete = TRUE;
						replace = TRUE;
						break;
					case 'P':	/* Give path to index files */
						strcpy (path, &argv[i][2]);
                        path_specified = 1;
						break;
					case 'V':
						verbose = TRUE;
						break;
					default:
						error = TRUE;
						break;
				}
			}
		}
		else
			fp = fopen(argv[i],"r");
	}
	
	if (error || fp == NULL || !path_specified) {
		fprintf(stderr, "Usage: binlegsplus leglistfile -P<Path> [-D] [-R] [-V]\n");
		fprintf(stderr,"	-P gives directory for the index files\n");
		fprintf(stderr,"	-D will remove info for the listed legs\n");
		fprintf(stderr,"	-R will replace old info. [Default is append].\n");
		fprintf(stderr,"	-V (verbose) will print update-messages. [Default is silent]\n");
		exit(-1);
	}

	for (i=0; i<14; i++) flag[i] = 0;
	
	info_head = make_info ("-", 0, 0, NULL);
	this_info = info_head;
	for (bin = 0; bin < 64800; bin++) grid[bin].n_legs = 0;

	/* path priority: 1) -P argument
	                  2) GMTPLUS_LIBDIR
                      3) default (LIBDIR)
	*/
	if (path[0]) {
		sprintf (leg_file, "%s/gmt_legs.d\0", path);
		sprintf (index_file, "%s/gmt_index.b\0", path);
	}
	else {
        /* Note: 08/03/95 this code will never be called because
                 it was decided that the path *must* be specified
                 with the -P option, and the program will exit with
                 a usage message before this if it is not.  But, the
                 code is left intact in case this decision is reversed
                 in the future.
        */
		envdir = getenv("GMTPLUS_LIBDIR");      
		if (envdir == NULL) {
			sprintf (leg_file, "%s/gmt_legs.d\0", LIBDIR);
			sprintf (index_file, "%s/gmt_index.b\0", LIBDIR);
		} else {
			sprintf (leg_file, "%s/gmt_legs.d\0", envdir);
			sprintf (index_file, "%s/gmt_index.b\0", envdir);
		}
	}
	
	
	if ((fleg = fopen (leg_file,"r")) == NULL) {
		fprintf (stderr,"Could not find %s\n", leg_file);
		exit(-1);
	}
	if ((fbin = fopen (index_file,"r")) == NULL) {
		fprintf (stderr,"Could not open %s\n", index_file);
		exit(-1);
	}
	
	last_id  = -1;
	
	/* Read existing leg-information from gmt_legs.d file */
	
	while (fgets (line, 512, fleg)) {
		num = sscanf (line, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
				lname, &id, &flag[0], &flag[1], &flag[2], &flag[3], &flag[4], 
				&flag[5], &flag[6], &flag[7], &flag[8], &flag[9], &flag[10], 
				&flag[11], &flag[12], &flag[13]);
		this_info->next_info = make_info (lname, id, num-2, flag);
		this_info = this_info->next_info;
		if (id > last_id) last_id = id;
	}
	fclose (fleg);
	
	/* Read geographical leg-info from gmt_index.b file */
	
	while ((fread ((char *)(&index), 4, 1, fbin)) == 1) {
		fread ((char *)(&no_of_legs), 4, 1, fbin);
		if (grid[index].n_legs == 0) {
			grid[index].first_leg = make_leg (0, 0, NULL);
			grid[index].last_leg = grid[index].first_leg;
		}
		for (i = 0; i < no_of_legs; i++) {
			fread ((char *)(&id), 4, 1, fbin);
			fread ((char *)(&num), 4, 1, fbin);
			for (j=0; j<num; j++)
				fread((char *)(&flag[j]), 4, 1, fbin);
			grid[index].last_leg->next_leg = make_leg (id, num, flag);
			grid[index].last_leg = grid[index].last_leg->next_leg;
			grid[index].n_legs++;
		}
	}
	fclose (fbin);
	
	/* Ok, now we can start reading new info */
	
	while (fgets (line, 512, fp)) {
		sscanf (line, "%s", leg);
		for (this_info = info_head; this_info->next_info && 
			strcmp (this_info->next_info->legname, leg) < 0;
			this_info = this_info->next_info);
		free_id = 0;
		found_it = (this_info->next_info != (struct INFO *)NULL);
		if (found_it && !strcmp (this_info->next_info->legname, leg)) {
			if (replace) {
				if (verbose) fprintf (stderr, "binlegsplus: Removing old info for leg: %s\n", leg);
				free_id = remove_leg (this_info->next_info->leg_id);
				if (delete) this_info->next_info = this_info->next_info->next_info;
			}
			else {
				if (verbose) fprintf (stderr, "binlegsplus: Leg %s already in system! (Skipped)\n",leg);
				continue;
			}
		}
		if (delete ) {	/* Give message and go back and read next leg */
			if (found_it)
				fprintf (stderr, "binlegsplus: Leg %s removed\n", leg);
			else
				fprintf (stderr, "binlegsplus: Leg %s not in system!\n", leg);
			continue;
		}
			
		/* Open the bix file */
		
		sprintf (file, "%s.bix\0", leg);
		if ((find = fopen (file,"r")) == NULL) {
			fprintf (stderr, "Could not find file: %s\n",file);
			continue;
		}
		if (verbose) fprintf (stderr, "binlegsplus: Adding leg: %s\n",leg);
		fgets (line, 512, find);
		num_max = sscanf (line, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d", 
			&year, &max[0], &max[1], &max[2], &max[3], &max[4], 
			&max[5], &max[6], &max[7], &max[8], &max[9], &max[10], 
			&max[11], &max[12], &max[13]);
		
		/* If a leg is replaced, then use the same id_no, else increment to get a new one */
		
		id = (free_id) ? free_id : ++last_id;
		if (!free_id) {
			/* allocate a new info struct and insert here */
			new_info = make_info (leg, id, num_max-1, max);
			new_info->next_info = this_info->next_info;
			this_info->next_info = new_info;
			this_info = new_info;
		}
		else {
			/* update the current info struct, leave in list */
			this_info = this_info->next_info;
			update_info(this_info, leg, id, num_max-1, max);
		}
		/* read in bin number plus that bin's flags...*/
		while (fgets (line, 512, find)) {
			num = sscanf (line, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d", 
				&index, &flag[0], &flag[1], &flag[2], &flag[3], &flag[4], 
				&flag[5], &flag[6], &flag[7], &flag[8], &flag[9], &flag[10], 
				&flag[11], &flag[12], &flag[13]);
			if (num < 2) {
				fprintf(stderr,"binlegsplus: gmt-flag no good! (Leg=%s)\n", leg);
				exit(-1);
			}
			if (grid[index].n_legs == 0) {
				grid[index].first_leg = make_leg (0, 0, NULL);
				grid[index].last_leg = grid[index].first_leg;
			}
			grid[index].last_leg->next_leg = make_leg (id, num-1, flag);
			grid[index].last_leg = grid[index].last_leg->next_leg;
			grid[index].n_legs++;
		}
		fclose (find);
	}
	fclose (fp);
	
	/* Done, now we must rewrite the gmt_index.b and gmt_legs.d files */
	
	sprintf (old_leg_file, "%s_old\0", leg_file);
	sprintf (old_index_file, "%s_old\0", index_file);
	
	if (rename (leg_file, old_leg_file)) {
		fprintf (stderr, "binlegsplus: Rename failed for %s. Aborts!\n", leg_file);
		exit (-1);
	}
	if (rename (index_file, old_index_file)) {
		fprintf (stderr, "binlegsplus: Rename failed for %s. Aborts!\n", index_file);
		exit (-1);
	}
	
	if ((fleg = fopen (leg_file, "w")) == NULL) {
		fprintf (stderr, "binlegsplus: Failed to create %s. Aborts!\n", leg_file);
		exit (-1);
	}
	if ((fbin = fopen (index_file, "w")) == NULL) {
		fprintf (stderr, "binlegsplus: Failed to create %s. Aborts!\n", index_file);
		exit (-1);
	}
	for (this_info = info_head->next_info; this_info; 
		this_info = this_info->next_info) {
			fprintf(fleg, "%s %d", this_info->legname, this_info->leg_id);
			ptr = this_info->flags;
			while (ptr != NULL) {
				fprintf(fleg," %d", ptr->data);
				ptr = ptr->next;
			}
			fprintf(fleg, "\n");
	}
	fclose (fleg);
	chmod (leg_file, S_RDONLY);
	
	for (bin = 0; bin < 64800; bin++) {
		if (grid[bin].n_legs > 0) {
			fwrite ((char *)(&bin), 4, 1, fbin);
			fwrite ((char *)(&grid[bin].n_legs), 4, 1, fbin);
			for (this_leg = grid[bin].first_leg->next_leg; this_leg; this_leg = this_leg->next_leg) {
				fwrite ((char *)(&this_leg->leg_id), 4, 1, fbin);
				num = 0;
				ptr = this_leg->flags;
				while (ptr != NULL) {
					num++;
					ptr = ptr->next;
				}
				fwrite ((char *)(&num), 4, 1, fbin);
				ptr = this_leg->flags;
				for (i=0;i<num;i++) {
					fwrite((char *)(&ptr->data), 4, 1, fbin);
					ptr = ptr->next;
				}
			}
				
		}
	}
	fclose (fbin);
	chmod (index_file, S_RDONLY);
	if (verbose) fprintf (stderr, "binlegsplus completed successfully\n");
	exit(0);
}

/* make and fill in leg structure */
struct LEG *make_leg (id, num_flags, flag)
int id;
int num_flags;
int *flag; {
	struct LEG *new;
	int i;
	bin_flag_ptr ptr;
	if ((new = (struct LEG *) malloc (sizeof (struct LEG))) == NULL) {
		fprintf (stderr, "malloc returned NULL for make_leg\n");
		exit(-1);
	}
	new->leg_id = id;
	new->flags = NULL;
	for (i=0; i<num_flags; i++) {
		if (i == 0) {
			ptr = new->flags = (bin_flag_ptr)calloc(1, sizeof(bin_flag_type));
		} else {
			ptr->next = (bin_flag_ptr)calloc(1, sizeof(bin_flag_type));
			ptr = ptr->next;
		}
		ptr->data = flag[i];
		ptr->next = NULL;
	}
	new->next_leg = 0;
	return (new);
}

int remove_leg (leg_id)
int leg_id; {
	/* Remove all traces of the leg with id leg_id from structures */
	struct LEG *leg, *skip_leg;
	int bin;
	bin_flag_ptr ptr, back_ptr;
	
	for (bin = 0; bin < 64800; bin++) {
		if (grid[bin].n_legs == 0) continue;	/* No legs crossed this bin */
		
		for (leg = grid[bin].first_leg; leg->next_leg && leg->next_leg->leg_id != leg_id; leg = leg->next_leg);	/* Finds the leg or end-of-list */

		if (leg->next_leg) {	/* Ok, found it. Lets remove it from the list */
			skip_leg = leg->next_leg;
			leg->next_leg = skip_leg->next_leg;
			skip_leg->next_leg = 0;
			grid[bin].n_legs--;
			if (leg->next_leg == NULL) grid[bin].last_leg = leg;
			back_ptr = skip_leg->flags;
			while (back_ptr != NULL) {
				ptr = back_ptr->next;
				GMT_free(back_ptr);
				back_ptr = ptr;
			}
			GMT_free ((void *) skip_leg);
			if (grid[bin].n_legs == 0) GMT_free ((void *)grid[bin].first_leg);
		}
	}
	return (leg_id);
}
