/*--------------------------------------------------------------------
 *    The GMTPLUS-system:	@(#)gmtplus_mgg.c	1.2	9/12/95
 *
 *  Based on...
 *    The GMT-system:	@(#)gmt_mgg.c	2.7  7/22/93
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * gmt_mgg.c is a collection of functions that are used by several of
 * the mgg GMT-SYSTEM programs. These functions perform:
 *	> Converting time from seconds to dates and back again
 *	> Find right path to a gmt-file
 *  > parsing header file and first data line
 *
 * Author:	Paul Wessel
 * Date:	13-JUL-1988
 * Updated:	13-JUN-1991 v2.0
 *
 * List of functions:
 *
 *	gmt_date		- Convert seconds to gmt date.
 *	gmt_init		- Initialize gmt time functions.
 *	gmt_time		- Convert gmt date to seconds.
 *	gmtpath_init		- Get paths to gmtfile direcories.
 *	gmtpath_func		- Return full path to a leg.
 *
 *  01/10/94 jcm GMT_Plus 
 *
 * GMTPlus version 03/10/95 jcm
 * Modifications made for gmtplus database system.
 *
 *  read_gmt_header			- Parse the header file.
 *  read_gmt_master_index	- Parse the master index file.
 *  read_first_gmt_datarec	- Parse the first record in the datafile.
 *  write_gmt_header		- Write header info to file.
 *  write_first_gmt_record	- Write the first record in the datafile.
 *  make_info				- Create and fill info structure for index files.	
 *  update_info				- Change index information in info structure.
 *  add_pedigree			- Add appropriate comment line to header file.
 *  transfer_header_comment	- Copy header comments to new header file.
 */

#include <stdlib.h> 
#include <gmt.h>	/* System dependent files */
#include "gmtplus_mgg.h"

char LIBDIR[PATH_MAX]; /* FIX: where was this declared before? */

char *GMT_path[50];	/* Max 50 directories for now */
int n_gmt_paths = 0;	/* Number of these directories */

/* GMT function gmt_date computes the date (hr/mi/sec/dd/mm/yy) based
 * on the total time in seconds since the beginning of first_year.
 * The pointer to the GMT structure is passed allong with the other
 * arguments. The Julian day is returned. the yymmddhhmmss is passed
 * through the argument list.
 */
 
int gmt_date(int time,int *year,int *month,int *day,int *hour,int *minute,int *second,struct GMT *GMT_struct)
{
	int day_time, julian_day;
	day_time = time/86400;
	*month = day_time / 31 + 1;
	if (GMT_struct->daymon[*month +1] <= day_time)
	  (*month)++;
	*year = (*month  - 1) / 12 + GMT_struct->first_year;
	*day = day_time - GMT_struct->daymon[*month] + 1;
	julian_day = (*month > 12) ?
		GMT_struct->daymon[*month] - GMT_struct->daymon[(*month - (*month)%12)] + *day :
		GMT_struct->daymon[*month] + *day;
	*month  = (*month-1)%12 + 1;
	time %= 86400;
	*hour = time / 3600;
	*minute = (time%3600) / 60;
	*second = time - *hour * 3600 - *minute * 60;
	return (julian_day);
}

/*
 *	GMT subroutine gmt_init sets up the structure GMT  which is
 *	used by other gmt routines (gmt_time,gmt_date) to convert
 *	times.  Daymon[month] contains the cumulative number of
 *	days from Jan 1 in first_year through the months PRIOR to the
 *	value of month.  0 <= month <= 60.  month = 0 only occurs
 *	during initializing in this routine. The user must declare
 *	a pointer to the struct GMT in the main program and pass it
 *	when calling the GMT_* functions. To define the GMT structure,
 *	include the file gmt.h
 *
 *	Paul Wessel
 *	12-JUL-1987
 *
 */
 
struct GMT *gmt_init(year1)
int year1; {
	struct GMT *GMT_struct;
	int dm[12];	/* No of days in each month */
	int year, this_year, month, m;
	GMT_struct = (struct GMT *) GMT_memory (CNULL, 1, sizeof(struct GMT), "gmt_init");
	GMT_struct->first_year = year1;
	/* initialize days of the month etc. */
	dm[0] = 0;
	dm[1] = 31;
	dm[2] = 28;
	dm[3] = 31;
	dm[4] = 30;
	dm[5] = 31;
	dm[6] = 30;
	dm[7] = 31;
	dm[8] = 31;
	dm[9] = 30;
	dm[10] = 31;
	dm[11] = 30;
	GMT_struct->daymon[0] = 0;
	for (year = 0, month = 0; year < 5; year++) {
		this_year = GMT_struct->first_year + year;
		if (this_year%4 == 0) dm[2] = 29;
		for (m = 1; m <= 12; m++) {
			month++;
			GMT_struct->daymon[month] = GMT_struct->daymon[month - 1] + dm[m - 1];
		}
   		dm[2] = 28;
   		dm[0] = 31;
   	}
   	return (GMT_struct);
}

/* GMT function gmt_time returns the number of seconds from
 * first_year calculated from (hr/mi/sc/dd/mm/yy). The pointer
 * to the GMT structure is passed along with the arguments.
 */
 /* MODIFIED 10 July, 1987 by W. Smith  --  I killed a bug in
     month calculation */
 
int gmt_time(int *time,int year,int month,int day,int hour,int minute,int second, struct GMT *GMT_struct)
{
	int mon, n_days;
	if ((mon = (year - GMT_struct->first_year)) > 4) {
	  fprintf(stderr,"gmt_time:  Year - first_year > 4\n");
	  return(-1);
	}
	mon = mon * 12 + month;
	n_days = GMT_struct->daymon[mon] + day - 1;
	*time = n_days * 86400 + hour * 3600 + minute * 60 + second;
	return (*time);
}
	
/* gmtpath_init reads the LIBDIR/gmtfile_paths file and gets all
 * the gmtfile directories.
 */
 
int gmtpath_init () {
  char file[512], line[512];
  FILE *fp;
  char *envdir;

	/* 07/28/94 jcm upgrade to use environment variables */
	/* order: GMTPLUS_LIBDIR, then default LIBDIR */
	envdir = getenv("GMTPLUS_LIBDIR");	
	if (envdir == NULL) {
		sprintf (file, "%s/gmtfile_paths\0", LIBDIR);
	} else {
		strcpy(file,envdir);
		strcat(file,"/gmtfile_paths");
	}
	
	n_gmt_paths = 0;

	if ((fp = fopen (file, "r")) == NULL) {
		fprintf (stderr, "GMTPLUS Warning: path file %s for *.gmt files not found\n", file);
		fprintf (stderr, "(Will only look in current directory for such files)\n");
		return (0);
	}
	
	while (fgets (line, 512, fp)) {
		if (line[0] == '#') continue;	/* Comments */
		if (line[0] == ' ' || line[0] == '\0') continue;	/* Blank line */
		GMT_path[n_gmt_paths] = GMT_memory (CNULL, 1, strlen (line), "gmtpath_init");
		line[strlen (line)-1] = 0;
		strcpy (GMT_path[n_gmt_paths], line);
		n_gmt_paths++;
	}
	fclose (fp);
}
	
/* gmtpath takes a legid as argument and returns the full path
 * to where this data file can be found.  gmtpath_init must be
 * called first
 */
/* FIX: doc return value meaning - is it 0=failed, 1=success?*/ 
int gmtpath_func (char *leg_path, char *head_path, char *leg)
{
	int id;
	char geo_path[512];
	char geo_head[512];
	
	/* First look in current directory */
	
	sprintf (geo_path, "%s.gmtd\0", leg);
	sprintf (geo_head, "%s.gmth\0", leg);
	if (!access(geo_path, R_OK) && !access(geo_head, R_OK)) {
		strcpy(leg_path, geo_path);
		strcpy(head_path, geo_head);
		return (0);
	}
	
	/* Then look elsewhere */
	
	for (id = 0; id < n_gmt_paths; id++) {
		sprintf (geo_path, "%s/%s.gmtd\0", GMT_path[id], leg);
		sprintf (geo_head, "%s/%s.gmth\0", GMT_path[id], leg);
		if (!access (geo_path, R_OK) && !access(geo_head, R_OK)) {
			strcpy (leg_path, geo_path);
			strcpy (head_path, geo_head);
			return (0);
		}
	}
	return(1);
}

/* read info from gmt header file & setup column structure */
int read_gmt_header(FILE *hp, struct COL_N *col, int *year, int *nrec, char *agency, char *cruise)
{
  char buffer[256];
  char tag[32];
  char data[80];
  int i;
  int col_data;
  int success;
  char file_name[80];
	


	col->tsec = 0;
	col->lat = 1;
	col->lon = 2;
	col->graw = 3;
	col->gfaa = 6;
	col->twtt = 7;
	col->tcor = 8;
	col->mtot = 9;
	col->manm = 10;
	col->mflt = -1;
    col->bad_flag = -1;
	col->user_long[0] = -1;
	col->user_long[1] = -1;
	col->user_long[2] = -1;
	col->user_long[3] = -1;
	col->user_long[4] = -1;
	col->user_long[5] = -1;
	col->user_short[0] = -1;
	col->user_short[1] = -1;
	col->user_short[2] = -1;
	col->user_short[3] = -1;
	col->user_short[4] = -1;
	col->user_short[5] = -1;
	col->user_short[6] = -1;
	col->user_short[7] = -1;

	sprintf(agency, "None");
	*year = 1994;
	*nrec = 10;

	while (fscanf(hp,"%[^\n]\n",buffer) != EOF) {
		if (!isspace(buffer[0]) && buffer[0] != '#') { /*i.e. not a comment field or blank */

			data[0] = '\0'; /*empty data string before reading in new line */

			sscanf(buffer,"%[^:]:%s",tag,data);
			if (strcmp(tag,"FILE_ID") == 0) {
				strcpy(file_name,data);
			}
			else if (strcmp(tag,"CRUISE") == 0) {
				strcpy(cruise,data);
			}
			else if (strcmp(tag,"NUMBER_OF_RECORDS") == 0) {
				sscanf(data,"%d",nrec);
			}
			else if (strcmp(tag,"AGENCY") == 0) {
				strcpy(agency,data);
			}
			else if (strcmp(tag,"LEG_YEAR") == 0) {
				sscanf(data,"%d",year);
			}
			else {
				success = FALSE;
				for (i=0;i<MAX_TYPES;i++) {
					if (strcmp(tag,gmt_column_name[i]) == 0) {
						success = TRUE;
						sscanf(data,"%d",&col_data);
						switch (i) {
							case TSEC:
								col->tsec = col_data;
								break;
							case LAT:
								col->lat = col_data;
								break;
							case LON:
								col->lon = col_data;
								break;
							case GRAW:
								col->graw = col_data;
								break;
							case TWTT:
								col->twtt = col_data;
								break;
							case TCORR:
								col->tcor = col_data;
								break;
							case GFAA:
								col->gfaa = col_data;
								break;
							case MTOT:
								col->mtot = col_data;
								break;
							case MANM:
								col->manm = col_data;
								break;
							case MFILT:
								col->mflt = col_data;
								break;
							case BAD_FLAG:
								col->bad_flag = col_data;
								break;
							case UL0:
								col->user_long[0] = col_data;
								break;
							case UL1:
								col->user_long[1] = col_data;
								break;
							case UL2:
								col->user_long[2] = col_data;
								break;
							case UL3:
								col->user_long[3] = col_data;
								break;
							case UL4:
								col->user_long[4] = col_data;
								break;
							case UL5:
								col->user_long[5] = col_data;
								break;
							case US0:
								col->user_short[0] = col_data;
								break;
							case US1:
								col->user_short[1] = col_data;
								break;
							case US2:
								col->user_short[2] = col_data;
								break;
							case US3:
								col->user_short[3] = col_data;
								break;
							case US4:
								col->user_short[4] = col_data;
								break;
							case US5:
								col->user_short[5] = col_data;
								break;
							case US6:
								col->user_short[6] = col_data;
								break;
							case US7:
								col->user_short[7] = col_data;
								break;
							default:
								fprintf(stderr,"Warning: invalid line found in header file.\n");
								break;
						}
						i = MAX_TYPES;
					}
				} 
				if (!success)
					fprintf(stderr,"Warning: invalid line found in header file.\n");
			} /* end else-if chain */

		} else { /* Is # or empty line */
			if (buffer[0] == '#') /*fprintf(stdout,"%s\n",buffer);*/
                ;
			else
				fprintf(stderr,"Warning: invalid line in header.\n");
		}
				
	} /* end while */


}

/*int read_gmt_master_index()*/
void read_gmt_master_index()
{
	int i;
	for (i=0;i<6;i++)
		col_short[i] = FALSE;
	for (i=6;i<14;i++)
		col_short[i] = TRUE;
	/* FIX: implicit returns is very bad!! Shame shame. */
}

int read_first_gmt_datarec(FILE *fp,struct COL_N *col)
{
	int i;
	int data;
	if (fread((char *)(&first_record), 40, 1, fp) != 1) {
		fprintf(stderr,"gmtplus: Error reading first data record\n");
		exit(-1);
	}
	
	for (i=0;i<6;i++) {
		data = first_record.ldata[i];
		switch (data) {
			case TSEC:
				col->tsec = i;
				break;
			case LAT:
				col->lat = i;
				break;
			case LON:
				col->lon = i;
				break;
			case GRAW:
				col->graw = i;
				break;
			case TWTT:
				col->twtt = i;
				break;
			case TCORR:
				col->tcor = i;
				break;
			case GFAA:
				col->gfaa = i;
				break;
			case MTOT:
				col->mtot = i;
				break;
			case MANM:
				col->manm = i;
				break;
			case MFILT:
				col->mflt = i;
				break;
			case BAD_FLAG:
				col->bad_flag = i;
				break;
			case UL0:
				col->user_long[0] = i;
				break;
			case UL1:
				col->user_long[1] = i;
				break;
			case UL2:
				col->user_long[2] = i;
				break;
			case UL3:
				col->user_long[3] = i;
				break;
			case UL4:
				col->user_long[4] = i;
				break;
			case UL5:
				col->user_long[5] = i;
				break;
			case US0:
				col->user_short[0] = i;
				break;
			case US1:
				col->user_short[1] = i;
				break;
			case US2:
				col->user_short[2] = i;
				break;
			case US3:
				col->user_short[3] = i;
				break;
			case US4:
				col->user_short[4] = i;
				break;
			case US5:
				col->user_short[5] = i;
				break;
			case US6:
				col->user_short[6] = i;
				break;
			case US7:
				col->user_short[7] = i;
				break;
			case -1:
				break;
			default:
				fprintf(stderr,"read_first_gmt_datarec: invalid data field listed\n");
				break;
		}
	}
	for (i=0;i<8;i++) {
		data = first_record.sdata[i];
		switch (data) {
			case TSEC:
				col->tsec = i+6;
				break;
			case LAT:
				col->lat = i+6;
				break;
			case LON:
				col->lon = i+6;
				break;
			case GRAW:
				col->graw = i+6;
				break;
			case TWTT:
				col->twtt = i+6;
				break;
			case TCORR:
				col->tcor = i+6;
				break;
			case GFAA:
				col->gfaa = i+6;
				break;
			case MTOT:
				col->mtot = i+6;
				break;
			case MANM:
				col->manm = i+6;
				break;
			case MFILT:
				col->mflt = i+6;
				break;
			case BAD_FLAG:
				col->bad_flag = i+6;
				break;
			case UL0:
				col->user_long[0] = i+6;
				break;
			case UL1:
				col->user_long[1] = i+6;
				break;
			case UL2:
				col->user_long[2] = i+6;
				break;
			case UL3:
				col->user_long[3] = i+6;
				break;
			case UL4:
				col->user_long[4] = i+6;
				break;
			case UL5:
				col->user_long[5] = i+6;
				break;
			case US0:
				col->user_short[0] = i+6;
				break;
			case US1:
				col->user_short[1] = i+6;
				break;
			case US2:
				col->user_short[2] = i+6;
				break;
			case US3:
				col->user_short[3] = i+6;
				break;
			case US4:
				col->user_short[4] = i+6;
				break;
			case US5:
				col->user_short[5] = i+6;
				break;
			case US6:
				col->user_short[6] = i+6;
				break;
			case US7:
				col->user_short[7] = i+6;
				break;
			case -1:
				break;
			default:
				fprintf(stderr,"read_first_gmt_datarec: invalid data field listed\n");
				break;
		}
	}
	/* FIX: Another nasty implicit return.  BAD BAD BAD */
}

int write_gmt_header(FILE *hp, struct COL_N *col, int year, int nrec, char *agency, char *cruise, char *filename)
{
	int i;

	fprintf(hp, "FILE_ID: %s\n", filename);
	fprintf(hp, "CRUISE: %s\n", cruise);
	fprintf(hp, "NUMBER_OF_RECORDS: %d\n", nrec);
	fprintf(hp, "AGENCY: %s\n", agency);
	fprintf(hp, "LEG_YEAR: %d\n", year);
	fprintf(hp, "%s: %d\n", gmt_column_name[TSEC],col->tsec);
	fprintf(hp, "%s: %d\n", gmt_column_name[LAT],col->lat);
	fprintf(hp, "%s: %d\n", gmt_column_name[LON],col->lon);
	fprintf(hp, "%s: %d\n", gmt_column_name[GRAW],col->graw);
	fprintf(hp, "%s: %d\n", gmt_column_name[GFAA],col->gfaa);
	fprintf(hp, "%s: %d\n", gmt_column_name[TWTT],col->twtt);
	fprintf(hp, "%s: %d\n", gmt_column_name[TCORR],col->tcor);
	fprintf(hp, "%s: %d\n", gmt_column_name[MTOT],col->mtot);
	fprintf(hp, "%s: %d\n", gmt_column_name[MANM],col->manm);
	fprintf(hp, "%s: %d\n", gmt_column_name[MFILT],col->mflt);
	fprintf(hp, "%s: %d\n", gmt_column_name[BAD_FLAG],col->bad_flag);
	for (i=0; i<6; i++) {
		fprintf(hp, "%s: %d\n", gmt_column_name[UL0+i], col->user_long[i]);
	}
	for (i=0; i<8; i++) {
		fprintf(hp, "%s: %d\n", gmt_column_name[US0+i], col->user_short[i]);
	}

}

int write_first_gmt_datarec(FILE *fp,struct COL_N *col)
{
	int i;
	int data;
	
	for (i=0;i<6;i++)
		first_record.ldata[i] = -1;
	for (i=0;i<8;i++)
		first_record.sdata[i] = -1;

	if (col->tsec != -1) {
		if (col_short[col->tsec]) 
			first_record.sdata[col->tsec-6] = TSEC;
		else
			first_record.ldata[col->tsec] = TSEC;
	}	
	if (col->lat != -1) {
		if (col_short[col->lat]) 
			first_record.sdata[col->lat-6] = LAT;
		else
			first_record.ldata[col->lat] = LAT;
	}	
	if (col->lon != -1) {
		if (col_short[col->lon]) 
			first_record.sdata[col->lon-6] = LON;
		else
			first_record.ldata[col->lon] = LON;
	}	
	if (col->graw != -1) {
		if (col_short[col->graw]) 
			first_record.sdata[col->graw-6] = GRAW;
		else
			first_record.ldata[col->graw] = GRAW;
	}	
	if (col->gfaa != -1) {
		if (col_short[col->gfaa]) 
			first_record.sdata[col->gfaa-6] = GFAA;
		else
			first_record.ldata[col->gfaa] = GFAA;
	}	
	if (col->twtt != -1) {
		if (col_short[col->twtt]) 
			first_record.sdata[col->twtt-6] = TWTT;
		else
			first_record.ldata[col->twtt] = TWTT;
	}	
	if (col->tcor != -1) {
		if (col_short[col->tcor]) 
			first_record.sdata[col->tcor-6] = TCORR;
		else
			first_record.ldata[col->tcor] = TCORR;
	}	
	if (col->mtot != -1) {
		if (col_short[col->mtot]) 
			first_record.sdata[col->mtot-6] = MTOT;
		else
			first_record.ldata[col->mtot] = MTOT;
	}	
	if (col->manm != -1) {
		if (col_short[col->manm]) 
			first_record.sdata[col->manm-6] = MANM;
		else
			first_record.ldata[col->manm] = MANM;
	}	
	if (col->mflt != -1) {
		if (col_short[col->mflt]) 
			first_record.sdata[col->mflt-6] = MFILT;
		else
			first_record.ldata[col->mflt] = MFILT;
	}	
	if (col->bad_flag != -1) {
		if (col_short[col->bad_flag]) 
			first_record.sdata[col->bad_flag-6] = BAD_FLAG;
		else
			first_record.ldata[col->bad_flag] = BAD_FLAG;
	}	
	for (i=0;i<6;i++) {
		if (col->user_long[i] != -1) {
			first_record.ldata[col->user_long[i]] = UL0+i;
		}
	}	
	for (i=0;i<8;i++) {
		if (col->user_short[i] != -1) {
			first_record.sdata[col->user_short[i]-6] = US0+i;
		}
	}	

	if (fwrite((char *)(&first_record), 40, 1, fp) != 1) {
		fprintf(stderr,"gmtplus: Error writing first data record\n");
		exit(-1);
	}

}

struct INFO *make_info (name, id_no, num_flags, flag)
char *name;
int id_no;
int num_flags;
int *flag; {
    int i;
    bin_flag_ptr ptr;
    struct INFO *new;
    if ((new = (struct INFO *) malloc (sizeof (struct INFO))) == NULL) {
        fprintf (stderr, "malloc returned NULL for make_info\n");
        exit(-1);
    }
    strcpy (new->legname, name);
    new->leg_id = id_no;
    new->flags = NULL;
    for (i=0; i<num_flags; i++) {
        if (i == 0) {
            ptr = new->flags = (bin_flag_ptr)calloc(1, sizeof(bin_flag_type));
        } else {
            ptr->next = (bin_flag_ptr)calloc(1, sizeof(bin_flag_type));
            ptr = ptr->next;
        }
        ptr->data = flag[i];
        ptr->next = NULL;
    }

    new->next_info = 0;
	new->mark = FALSE;
    return (new);
}

/* update_info - replace contents of  info structure with
   (potentially) new id, leg name, and flags.
   DO NOT replace "next" flag, as this struct may be in
   a linked list and we want it to remain there.
*/ 
void update_info (struct INFO *old_info, char *name, int id_no, int num_flags, int *flag)
{
    int i;
    bin_flag_ptr ptr, next;

    strcpy (old_info->legname, name);
    old_info->leg_id = id_no;
    ptr = old_info->flags;
    while (ptr->next != NULL) {
       next = ptr->next;
       free(ptr);
       ptr = next;
    }
    if (ptr != NULL) free(ptr);
    old_info->flags = NULL;
    for (i=0; i<num_flags; i++) {
        if (i == 0) {
            ptr = old_info->flags = (bin_flag_ptr)calloc(1, sizeof(bin_flag_type));
        } else {
            ptr->next = (bin_flag_ptr)calloc(1, sizeof(bin_flag_type));
            ptr = ptr->next;
        }
        ptr->data = flag[i];
        ptr->next = NULL;
    }
    old_info->mark = FALSE;
    return; /* FIX: Is this really a void function? */
}


/* Add a comment to the header file pedigree section */ 
int add_pedigree(FILE *hp, const char *message)
{
  int r = fprintf(hp, "#%s\n", message);
  return (r);
}

int transfer_header_comments(FILE *from, FILE *to)
{
   int count;
   FILE *tpo;
   char tempfile[80];
   char comment[120];


   while ((count = fscanf(from,"%[^\n]\n",comment)) != EOF
           && (count != 0)) {
      if (comment[0] == '#')
         fprintf(to,"%s\n",comment);
   }

}

#define TWO_ZERO 1 
#define TWO_ONE 2
#define TWO_TWO 4
#define TWO_THREE 8
#define TWO_FOUR 16
#define TWO_FIVE 32
#define TWO_SIX 64
#define TWO_SEVEN 128
#define TWO_EIGHT 256
#define TWO_NINE 512
#define TWO_TEN 1024
#define TWO_ELEVEN 2048
#define TWO_TWELVE 4096
#define TWO_THIRTEEN 8192
#define NODATA -32000

int read_data_record(struct GMT_PLUS_REC *record, struct COL_N *col, FILE *fp)
{
   int flag;

   if (fread((char *)record, 40, 1, fp) != 1) return -1;
   if (col->bad_flag != -1) {
      flag = record->sdata[col->bad_flag];
      if (flag & TWO_ZERO) record->ldata[0] = NODATA;
      if (flag & TWO_ONE) record->ldata[1] = NODATA;
      if (flag & TWO_TWO) record->ldata[2] = NODATA;
      if (flag & TWO_THREE) record->ldata[3] = NODATA;
      if (flag & TWO_FOUR) record->ldata[4] = NODATA;
      if (flag & TWO_FIVE) record->ldata[5] = NODATA;
      if (flag & TWO_SIX) record->sdata[0] = NODATA;
      if (flag & TWO_SEVEN) record->sdata[1] = NODATA;
      if (flag & TWO_EIGHT) record->sdata[2] = NODATA;
      if (flag & TWO_NINE) record->sdata[3] = NODATA;
      if (flag & TWO_TEN) record->sdata[4] = NODATA;
      if (flag & TWO_ELEVEN) record->sdata[5] = NODATA;
      if (flag & TWO_TWELVE) record->sdata[6] = NODATA;
      if (flag & TWO_THIRTEEN) record->sdata[7] = NODATA;
	}
	return 0;
}

int data_is_flagged(flag, col_num)
int flag;
int col_num;
{
   if (flag == 0) return 0;
   switch (col_num) {
      case 0:
         if (flag & TWO_ZERO) return 1; else return 0;
         break;
      case 1:
         if (flag & TWO_ONE) return 1; else return 0;
         break;
      case 2:
         if (flag & TWO_TWO) return 1; else return 0;
         break;
      case 3:
         if (flag & TWO_THREE) return 1; else return 0;
         break;
      case 4:
         if (flag & TWO_FOUR) return 1; else return 0;
         break;
      case 5:
         if (flag & TWO_FIVE) return 1; else return 0;
         break;
      case 6:
         if (flag & TWO_SIX) return 1; else return 0;
         break;
      case 7:
         if (flag & TWO_SEVEN) return 1; else return 0;
         break;
      case 8:
         if (flag & TWO_EIGHT) return 1; else return 0;
         break;
      case 9:
         if (flag & TWO_NINE) return 1; else return 0;
         break;
      case 10:
         if (flag & TWO_TEN) return 1; else return 0;
         break;
      case 11:
         if (flag & TWO_ELEVEN) return 1; else return 0;
         break;
      case 12:
         if (flag & TWO_TWELVE) return 1; else return 0;
         break;
      case 13:
         if (flag & TWO_THIRTEEN) return 1; else return 0;
         break;
   }
   return 1;
}
