/*--------------------------------------------------------------------
 *    The GMTPLUS-system:	@(#)gmtpluslist.c	1.2	9/12/95
 *
 *  Based on...
 *    The GMT-system:	@(#)gmtlist.c	2.7  7/22/93
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * gmtlist produces ASCII listings of <legid>.gmt files. The *.gmt files
 * contains time(s), latitude(y), longitude(x), gravity(g), magnetics(m),
 * and bathymetry(t), and the user may extract any combination of these 6
 * parameters + distance (in km), heading, velocity, and weight by using the option
 * -Fsyxgmtdhvw. The sequence
 * in which the flag characters appear determines the sequence in which the
 * parameters will be printed out. If no options is specified, the default
 * is -sxygmtd.  E.g. to create an input file for surface, use -xyg (for gravity).
 * If upper case letters are used for gmt (GMT), then only records that have
 * that particular data are written out.  E.g -xyg gives
 * lon/lat/grav, whereas -xyG gives lon/lat/grav where there is gravity data.
 * To select a section of the track, specify the start/endpoints by:
 *	1) Start-time (yyyy/mm/dd/hh:mm) OR start-distance (km)
 *	2) Stop-time (yyyy/mm/dd/hh:mm) OR stop-distance (km)
 * To select data inside an area, use the -R option.
 * To create a Scientist Helper Ascii Table format file, use -H.
 * Several formats for time is available. The default when using -s is
 * seconds from Jan 1 the year the cruise started. -sc (calender) gives
 * yymmddhhmms output, and -sj (julian) gives yyjjhhmmss output.
 *
 * Author:	Paul Wessel
 * Date:	19-APR-1988
 * Version:	2.1 1-JUL-1992
 *
 *
 * 01/11/94 jcm GMT_Plus
 * 02/24/94 jcm crosstrack corrections ignored for the time being.
 *
 *
 * GMTPlus version 03/10/95 jcm
 * Modifications made for gmtplus database system.
 *
 */
 
#include "gmt.h"
#include "gmtplus_mgg.h"
#include "x_system.h"

#define KMPRDEG 111.1949e-6
#define RADPRDEG 0.01745329e-6
#define MAXLEGS 5000
#define S_PR_DAY 86400

	
struct CORR **bin;

int binsize = sizeof(struct CORR);
int nlegs = 0;


int get_id (const char *name)
{
  int left, right, mid, cmp;
  
  left = 0;
  right = nlegs-1;
  while (left <= right) {
    mid = (left + right)/2;
    cmp = strcmp(name, bin[mid]->name);
    if (cmp < 0)
      right = mid-1;
    else if (cmp > 0)
      left = mid+1;
    else
      return (mid);
  }
  return (-1);
}


int
main (int argc, char *argv[])
{
	int leg_year, rec, i, j, n_records, argno, id, no[10], nval = 0;
	int start_time = 0, stop_time = 2000000000, dlon, last_lon;
	int mon1, day1 = 0, year1, hour1, min1, time, last_lat, mon2, day2 = 0, year2, hour2, min2;
	int yy, mm, dd, hh, mi, ss, jd, dt, last_time, n_cruises = 0;
	
	int error = FALSE, year_error = FALSE, wantgmt, want_all = FALSE, geodetic = TRUE;
	int correct = FALSE, tsec = FALSE, calender = FALSE, do_heading = FALSE;
	int no_g, no_m, no_t, greenwich = FALSE, do_speed = FALSE, speedknots = FALSE;
	int len;
	int splitdata = FALSE;
	int	plusminus180range = FALSE, dateline = FALSE;
	
	char gmtfile[80], gmtheader[80], corrfile[80], agency[10] = "", ndata, g, m, t;
	char cruise[80] = "";
	
	double lat, lon ,grv, mag, top, dist, start_dist = 0., stop_dist = 1.0E100;
	double ds, dx, dy, west = 0.0, east = 360.0, south = -90.0, north = 90.0;
	double heading, speed, weight = 1.0;
        double us_save;
	int tot_cols = 19;
	int missing, column;
	int n_cols_out, col_out[19], data[19], req[19];

	struct GMT *gmt;
	
	FILE *fp, *hp, *fpc;
	
	struct GMT_PLUS_REC record;
	struct COL_N col;
	
	for (i=0; i<tot_cols; i++) {
		req[i] = FALSE;
		data[i] = NODATA;
		col_out[i] = 0;
	}
	n_cols_out = 0;

	read_gmt_master_index();
	
	/* Check and interpret the command line arguments */
	
	for (i =1; !error && i < argc; i++) {
		if (argv[i][0] == '-') {
			switch(argv[i][1]) {
			
				case 'H':
				case 'R':
				case 'V':
				case '\0':
					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
					break;
					
				case 'C':
					if (argv[i][2] == 0)	/* Use default corrfile */
						sprintf (corrfile, "%s/xx_corrections.b\0", LIBDIR);
					else
						strcpy (corrfile, &argv[i][2]);
					correct = TRUE;
					break;
					
				case 'D':		/* Assign start/stop times for sub-section */
					if (argv[i][2] == 'a') {	/* Start date */
						sscanf(&argv[i][3], "%d/%d/%d/%d:%d",
							&year1, &mon1, &day1, &hour1, &min1);
						if (year1 < 1900) year_error = TRUE;
					}
					else if (argv[i][2] == 'b')	 {	/* Stop date */
						sscanf(&argv[i][3], "%d/%d/%d/%d:%d",
							&year2, &mon2, &day2, &hour2, &min2);
						if (year2 < 1900) year_error = TRUE;

					}
					else
						error = TRUE;
					break;

				case 'F':	/* Selected output fields */
					len = strlen(argv[i]);
					for (j = 2; j<len; j+=2) { /* skip over ':' */
						switch (argv[i][j]) {
							case 'G':	/* Records with gravity != NODATA requested */
								req[n_cols_out] = TRUE;
	       					case 'g':		/* Gravity is requested */
								switch (argv[i][j+1]) {
									case 'R':
									case 'r':
										col_out[n_cols_out++] = GRAW;
										break;
									case 'F':
									case 'f':
										col_out[n_cols_out++] = GFAA;
										break;
									default:
										error = TRUE;
										break;
								}
								j++;
								break;
							case 'M':	/* Records with magnetics != NODATA requested */
								req[n_cols_out] = TRUE;
							case 'm':		/* Magnetics is requested */
								switch (argv[i][j+1]) {
									case 'T':
									case 't':
										col_out[n_cols_out++] = MTOT;
										break;
									case 'A':
									case 'a':
										col_out[n_cols_out++] = MANM;
										break;
									case 'F':
									case 'f':
										col_out[n_cols_out++] = MFILT;
										break;
									default:
										error = TRUE;
										break;
								}
								j++;
								break;
							case 'T':	/* Records with topo != NODATA requested */
								req[n_cols_out] = TRUE;
							case 't':		/* Topography is requested */
								switch (argv[i][j+1]) {
									case 'T':
									case 't':
										col_out[n_cols_out++] = TWTT;
										break;
									case 'C':
									case 'c':
										col_out[n_cols_out++] = TCORR;
										break;
									default:
										error = TRUE;
										break;
								}
								j++;
								break;
							case 'x':		/* Longitude is requested */
								col_out[n_cols_out++] = LON;
								break;
							case 'y':		/* Latitude is requested */
								col_out[n_cols_out++] = LAT;
								break;
							case 's':		/* Time (in sec) is requested */
								col_out[n_cols_out++] = TSEC;
								if (argv[i][j+1] == 'c') {
									calender = TRUE;
									j++;
								}
								else if (argv[i][j+1] == 'j')
									j++;
								else
									tsec = TRUE; /* 02/24/94 jcm hmm?? */
								break;
							case 'd':		/* Distance (in km) is requested */
								col_out[n_cols_out++] = DISTANCE;
								break;
							case 'h':		/* Heading is requested */
								col_out[n_cols_out++] = HEADING;
								do_heading = TRUE;
								break;
							case 'v':		/* ship speed (in m/s) is requested */
								col_out[n_cols_out++] = VELOCITY;
								do_speed = TRUE;
								if (argv[i][j+1] == 'k') {
									speedknots = TRUE;	/* speed in knots */
									j++;
								}
								break;
							case 'w':		/* weights (Set with -W) is requested */
								col_out[n_cols_out++] = WEIGHT;
								break;
							case 'i':		/* Cruise leg ID is requested */
								col_out[n_cols_out++] = CRUISELEG_ID;
								break;
							case 'U':
								req[n_cols_out] = TRUE;
							case 'u':
								switch (argv[i][j+1]) {
									case 'L':
									case 'l':
										switch (argv[i][j+2]) {
											case '0':
												col_out[n_cols_out++] = UL0;
												break;
											case '1':
												col_out[n_cols_out++] = UL1;
												break;
											case '2':
												col_out[n_cols_out++] = UL2;
												break;
											case '3':
												col_out[n_cols_out++] = UL3;
												break;
											case '4':
												col_out[n_cols_out++] = UL4;
												break;
											case '5':
												col_out[n_cols_out++] = UL5;
												break;
											default:
												error = TRUE;
												break;
										}
										break;
									case 'S':
									case 's':
										switch (argv[i][j+2]) {
											case '0':
												col_out[n_cols_out++] = US0;
												break;
											case '1':
												col_out[n_cols_out++] = US1;
												break;
											case '2':
												col_out[n_cols_out++] = US2;
												break;
											case '3':
												col_out[n_cols_out++] = US3;
												break;
											case '4':
												col_out[n_cols_out++] = US4;
												break;
											case '5':
												col_out[n_cols_out++] = US5;
												break;
											case '6':
												col_out[n_cols_out++] = US6;
												break;
											case '7':
												col_out[n_cols_out++] = US7;
												break;
											default:
												error = TRUE;
												break;
										}
										break;
									default:
										error = TRUE;
										break;
								}
								j += 2;
								break;
											
							default:
								error = TRUE;
								break;
						}
					}
					break;
						
				case 'S':		/* Assign start/stop position for sub-section */
					if (argv[i][2] == 'a')	/* Start position */
						start_dist = atof(&argv[i][3]);
					else if (argv[i][2] == 'b')	/* Stop position */
						stop_dist = atof(&argv[i][3]);
					else
						error = TRUE;
					break;
					
				case 'G':
					geodetic = FALSE;
					break;
				case 'W':		/* Assign a weight to these data */
					weight = atof (&argv[i][2]);
					break;
				case 'X':       /* Separate cruises with > to facilitate psxy */
					splitdata = TRUE;
					break;
					
				default:		/* Options not recognized */
					error = TRUE;
					break;
			}
		}
		else
			n_cruises++;
	}
	
	if ((east < west) && (east < 0.0)) {
		east += 360.0; 
		dateline = TRUE;
	}

	/* Check that the options selected are mutually consistant */
	
	error += GMT_check_region (west, east, south, north);
	if (n_cols_out > tot_cols) error = TRUE;
	if (start_dist > stop_dist || start_time > stop_time) error = TRUE;
	if ((day1 > 0 && start_dist > 0.) || (day2 > 0 && stop_dist < 1.0e100)) error = TRUE;
	if (east < west || south > north) error = TRUE;
	if (n_cruises == 0) error = TRUE;
	if (weight <= 0.0) error = TRUE;

	if (year_error) {

		fprintf(stderr,"\n\tError: Enter full year in -D option, for example, 1967 instead of just 67\n\n");
		exit(-1);
	}

	if (error || argc == 1) {	/* Display usage */
		fprintf(stderr,"usage: gmtpluslist <cruise(s)> [-C<corrfile>] [-Da<startdate>] [-Db<stopdate>] [-F<dataflags>]\n");
		fprintf(stderr,"	[-G] [-H] [-R<west>/<east>/<south>/<north>] [-Sa<startdist>] [-Sb<stopdist>] [-V] [-W<Weight>] [-X] [-I]\n\n");
         
		/*if (GMT_quick) exit (-1);*/
              
		fprintf(stderr,"	<cruises> is one or more legnames, e.g. c2104 v3206 etc.\n");
		fprintf(stderr,"	OPTIONS:\n\n");
		fprintf(stderr,"	-C<file> applies crossover corrections to the data. If no file name\n");
		fprintf(stderr,"	   is given, the default correction file is assumed\n");
		fprintf(stderr,"	-Da<date> lists from date (given as yyyy/mm/dd/hh:mm)\n");
		fprintf(stderr,"	-Db<date> lists up to date (given as yyyy/mm/dd/hh:mm)\n");
		fprintf(stderr,"	-F Dataflags is a string made up of 1 or more of these characters:\n");
		fprintf(stderr,"	(separate each by a colon (:))\n");
		fprintf(stderr,"	  s means list time in seconds, sc gives dates, sj gives Julian day\n");
		fprintf(stderr,"	  x means list longitude (degrees)\n");
		fprintf(stderr,"	  y means list latitude (degrees)\n");
		fprintf(stderr,"	  gr means list raw gravity (mGal)\n");
		fprintf(stderr,"	  gf means list free-air gravity anomaly (mGal)\n");
		fprintf(stderr,"	  mt means list total magnetics (nTesla)\n");
		fprintf(stderr,"	  ma means list magnetic anomaly (nTesla)\n");
		fprintf(stderr,"	  mf means list filtered magnetics (nTesla)\n");
		fprintf(stderr,"	  tt means list two-way travel time (msec)\n");
		fprintf(stderr,"	  tc means list corrected depth (m)\n");
		fprintf(stderr,"	  d means list distance (km)\n"); 
		fprintf(stderr,"	  h means list heading (Degrees east from north)\n");
		fprintf(stderr,"	  v means list ship speed (m/s), vk gives speed in knots\n");
		fprintf(stderr,"	  w means list weight (see -W)\n");
		fprintf(stderr,"	  i means list cruise ID\n");
		fprintf(stderr,"	  ul0 - ul5 means list the data stored in long data 0 through data 5\n");
		fprintf(stderr,"	  us0 - us7 means list the data stored in short data 0 through data 7\n");
		fprintf(stderr,"	  If G, M, T or U is used instead of g, m, t or u, then only the records\n");
		fprintf(stderr,"	  that have that combination of data will be listed\n");
		fprintf(stderr,"	  The data is written out in the order specified in <dataflags>\n");
		fprintf(stderr,"	  [Default is -Fs:y:x:gr:gf:mt:ma:tt:tc:d:h:v:i and all records]\n");
		fprintf(stderr,"	-G force geographical longitudes (-180/+180) [Default is 0-360]\n");
		fprintf(stderr,"	-H write header record\n");
		fprintf(stderr,"	-R only return data inside the specified region\n");
		fprintf(stderr,"	-Sa<dist> lists from dist (in km)\n");
		fprintf(stderr,"	-Sb<dist> lists up to dist (in km)\n");
		fprintf(stderr,"	-V verbose, report progress\n");
		fprintf(stderr,"	-W sets weight for these data\n");
		fprintf(stderr,"	-X separates each cruise with '>' to facilitate psxy plotting\n");
		exit(-1);
	}

	gmtpath_init();

	if ((west < 0.0 && east > 0.0) || (west < 360.0 && east > 360.0)) greenwich = TRUE;
	if (!geodetic) greenwich = TRUE;

	if (west < 0.0) plusminus180range = TRUE;
	if (plusminus180range) {
		west += 360.0;
		east += 360.0;
	}
	
	if (correct) {	/* Read correction table */
		if ((fpc = fopen(corrfile,"r")) == NULL) {
			fprintf(stderr, "Could not read correction file %s\n", corrfile);
			exit(-1);
		}
		bin = (struct CORR **) malloc(sizeof(struct CORR *)*MAXLEGS);
		i = 0;
		bin[i] = (struct CORR *) malloc(sizeof(struct CORR));
		while (fread((char *)bin[i], binsize, 1, fpc) == 1) {
			i++;
			bin[i] = (struct CORR *) malloc(sizeof(struct CORR));
		}
		fclose(fpc);
		nlegs = i;
	}
			
		
	/* Sort the  order in which the parameters appear */
	
	if (n_cols_out == 0) {		/* Nothing selected, default used */
		g = m = t = TRUE;	/* No data was specified so all data [default] is output */
		for (i = 0; i < tot_cols; i++) col_out[i] = i;
		n_cols_out = tot_cols;
		want_all = tsec = TRUE;
		do_heading = do_speed = TRUE;
	}

	 
	if (gmtdefs.io_header) {
		/* Write out header record */
		for (i = 0; i < n_cols_out; i++) {
			switch (col_out[i]) {
				case TSEC:	/* Print out time header */
					if (tsec)
						printf("time(s)");
					else if (calender)
						printf("yy\tmm\tdd\thh\tmm\tss");
					else
						printf("yy\tjd\thh\tmm\tss");
					break;
				case LON:	/* Print out longitude */
					printf("lon");
					break;
				case LAT:	/* Print out latitude */
					printf("lat");
					break;
				case GFAA:	/* Print out gravity */
		 			printf("gfaa(mGal)");
		 			break;
				case GRAW:	/* Print out gravity */
		 			printf("graw(mGal)");
		 			break;
				case MTOT:	/* Print out magnetics */
		 			printf("mtot(nTesla)");
					break;
				case MANM:	/* Print out magnetics */
		 			printf("manm(nTesla)");
					break;
				case MFILT:	/* Print out magnetics */
		 			printf("mfilt(nTesla)");
					break;
				case TWTT:	/* Print out bathymetry */
					printf("twtt(msec)");
					break;
				case TCORR:	/* Print out bathymetry */
					printf("tcorr(m)");
					break;
				case DISTANCE:	/* Print out distance */
					printf("dist(km)");
					break;
				case HEADING:	/* Print out heading */
					printf("heading");
					break;
				case VELOCITY:	/* Print out ship speed */
					if (speedknots)
						printf("speed(knots)");
					else printf("speed(m/s)");
					break;
				case WEIGHT:	/* Print out weights */
					printf("weight");
					break;
				case CRUISELEG_ID:	/* Print out cruise id */
					printf("cruise");
					break;
				case UL0:
					printf("ul[0]");
					break;
				case UL1:
					printf("ul[1]");
					break;
				case UL2:
					printf("ul[2]");
					break;
				case UL3:
					printf("ul[3]");
					break;
				case UL4:
					printf("ul[4]");
					break;
				case UL5:
					printf("ul[5]");
					break;
				case US0:
					printf("us[0]");
					break;
				case US1:
					printf("us[1]");
					break;
				case US2:
					printf("us[2]");
					break;
				case US3:
					printf("us[3]");
					break;
				case US4:
					printf("us[4]");
					break;
				case US5:
					printf("us[5]");
					break;
				case US6:
					printf("us[6]");
					break;
				case US7:
					printf("us[7]");
					break;
			}
			((i+1) < n_cols_out) ? printf("\t") : printf("\n");
		}
	}

	for (argno = 1; argno < argc; argno++) {	/* Loop over all the files */
	
		if (argv[argno][0] == '-') continue;
  		if (gmtpath_func (gmtfile, gmtheader, argv[argno])) {
   			fprintf (stderr, "gmtpluslist : Cannot find leg %s\n", argv[argno]);
     			continue;
  		}
		if ((fp = fopen (gmtfile,"r")) == NULL) {
			fprintf (stderr,"gmtpluslist: Could not open %s\n", gmtfile);
			continue;
		}
		if ((hp = fopen (gmtheader,"r")) == NULL) {
			fprintf (stderr,"gmtpluslist: Could not open %s\n", gmtheader);
			fclose(fp);
			continue;
		}

		if (gmtdefs.verbose) fprintf (stderr, "gmtpluslist: Now processing cruise %s\n",  argv[argno]);
		
		/* read header file */
		read_gmt_header(hp, &col, &leg_year, &n_records, agency, cruise); 
		/* read first record in data file */
		read_first_gmt_datarec(fp, &col);
		/* begin reading data */
 	
		if (leg_year < 1900) leg_year += 1900;

		gmt = gmt_init(leg_year);	/* Initialize gmt_structure */
		if (correct)
			id = get_id(argv[argno]);
		else
			id = -1;

		/* Decode date to time in sec if needed */
	
		if (day1 > 0) gmt_time(&start_time, year1, mon1, day1, hour1, min1, 0, gmt);
		if (day2 > 0) gmt_time(&stop_time, year2, mon2, day2, hour2, min2, 0, gmt);
	
		dist = 0.0;
		time = 0;
	
		wantgmt = (g || m || t) ? TRUE : FALSE;
		if (want_all) wantgmt = FALSE;
		

		/* Start reading data from file */
	
		for (rec = 0; rec < n_records && dist < stop_dist && time < stop_time; rec++) {
			/*if (fread((char *)(&record), 40, 1, fp) != 1) {*/
			/* 08/31/95 jcm filter out flagged data when reading */
			if (read_data_record(&record, &col, fp) == -1) {
				fprintf(stderr,"gmtpluslist: Error reading data record no %d\n",rec);
				exit(-1);
			}
		
			/* Compute accumulated distance along track (Flat Earth) */
		
			if (rec == 0) {
				last_lon = record.ldata[col.lon];
				last_lat = record.ldata[col.lat];
				last_time = record.ldata[col.tsec];
				ds = dt = 0.0;
				heading = speed = NODATA;
			}
			else {
				dlon = record.ldata[col.lon] - last_lon;
				if (abs (dlon) > 180000000) dlon = copysign ((double) (360000000 - abs (dlon)), (double)dlon);
				dx = (double) dlon * cos (0.5*RADPRDEG*(double)(record.ldata[col.lat]+last_lat)) * KMPRDEG;
				dy = (double) (record.ldata[col.lat] - last_lat) * KMPRDEG;
				ds = hypot (dx, dy);
				dt = record.ldata[col.tsec] - last_time;
				if (do_heading) {
					heading = (dx == 0.0 && dy == 0.0) ? NODATA : 90.0 - R2D * atan2 (dy, dx);
					if (heading < 0.0) heading += 360.0;
				}
				if (do_speed) speed = (dt == 0) ? NODATA : 1000.0 * ds / dt;
				last_lon = record.ldata[col.lon];
				last_lat = record.ldata[col.lat];
				last_time = record.ldata[col.tsec];
			}
			dist += ds;
			
			/* Check if time or dist falls outside specified range */
		
			if (dist < start_dist) continue;
			if (dist > stop_dist) continue;
			if (record.ldata[col.tsec] < start_time) continue;
			if (record.ldata[col.tsec] > stop_time) continue;
		
			time = record.ldata[col.tsec];
			lat = (double) record.ldata[col.lat]*0.000001;
			lon = (double) record.ldata[col.lon]*0.000001;
		
			/* Check if lat/lon is outside specified area */
		
			if (lat < south || lat > north) continue;
			while (lon < west) lon += 360.0;
			if (lon > east) continue;
			while (lon > 360.0) lon -= 360.0;

			/* set up for required data */
			missing = FALSE;
			for (i=0; i<n_cols_out; i++) {
				if (req[i]) {
					switch (col_out[i]) {
						case GRAW:
							column = col.graw;
							break;
						case GFAA:
							column = col.gfaa;
							break;
						case TWTT:
							column = col.twtt;
							break;
						case TCORR:
							column = col.tcor;
							break;
						case MTOT:
							column = col.mtot;
							break;
						case MANM:
							column = col.manm;
							break;
						case MFILT: 
							column = col.mflt;
							break;
						case UL0:
							column = 0;
							break;
						case UL1:
							column = 1;
							break;
						case UL2:
							column = 2;
							break;
						case UL3:
							column = 3;
							break;
						case UL4:
							column = 4;
							break;
						case UL5:
							column = 5;
							break;
						case US0:
							column = 6;
							break;
						case US1:
							column = 7;
							break;
						case US2:
							column = 8;
							break;
						case US3:
							column = 9;
							break;
						case US4:
							column = 10;
							break;
						case US5:
							column = 11;
							break;
						case US6:
							column = 12;
							break;
						case US7:
							column = 13;
							break;
					}
					if (column == -1) {
						missing = TRUE;
					} else {
						if (col_short[column]) {
							if (record.sdata[column-6] == NODATA)
								missing = TRUE;
						} else {
							if (record.ldata[column] == NODATA)
								missing = TRUE;
						}
					}
					/*missing = (column == -1) ||
								(col_short[column]) ?
									(record.sdata[column-6] == NODATA)
									: (record.ldata[column] == NODATA);
					*/
				}
			}

			/* check for required data */
			if (missing) continue;
			/* This record will now be printed out */
		
			for (i = 0; i < n_cols_out; i++) {
				switch(col_out[i]) {
					case TSEC:	
						if (tsec)
							printf("%ld", record.ldata[col.tsec]);
						else if (calender) {
							gmt_date (record.ldata[col.tsec],&yy,&mm,&dd,&hh,&mi,&ss,gmt);
							printf ("%d\t%02d\t%02d\t%02d\t%02d\t%02d",
								yy, mm, dd, hh, mi, ss);
						}
						else {
							jd = gmt_date (record.ldata[col.tsec],&yy,&mm,&dd,&hh,&mi,&ss,gmt);
							printf ("%d\t%3d\t%2d\t%2d\t%2d",
								yy, jd, hh, mi, ss);
						}
						break;
					case LON:
						if (lon > 180.0 && greenwich) lon -= 360.0;
						if (lon > 180.0 && plusminus180range) lon -= 360.0;
						if (lon > 180.0 && dateline) lon -= 360.0;
						(gmtdefs.io_header) ? printf("%.5lf", lon) : printf("%9.5lf", lon);
						break;
					case LAT:	
						(gmtdefs.io_header) ? printf("%.5lf", lat) : printf("%8.5lf", lat);
						break;
					case GRAW:	
						if (col.graw == -1) grv = NODATA;
						else
						(col_short[col.graw]) ? (grv = (double)(record.sdata[col.graw-6]))
									: (grv = (double)(record.ldata[col.graw]));
						if (grv == NODATA)
							printf ("NaN");
						else {
							grv = 0.1*grv;
		 					(gmtdefs.io_header) ? printf("%.2lf", grv) : printf("%9.2lf", grv);
						}
		 				break;
					case GFAA:	
						if (col.gfaa == -1) grv = NODATA;
						else
						(col_short[col.gfaa]) ? (grv = record.sdata[col.gfaa-6])
									: (grv = record.ldata[col.gfaa]);
						if (grv == NODATA)
							printf ("NaN");
						else {
							grv = 0.1*grv;
		 					(gmtdefs.io_header) ? printf("%.2lf", grv) : printf("%9.2lf", grv);
						}
		 				break;
					case MTOT:	
						if (col.mtot == -1) mag = NODATA;
						else
						(col_short[col.mtot]) ? (mag = record.sdata[col.mtot-6])
									: (mag = record.ldata[col.mtot]);
						if (mag == NODATA)
							printf ("NaN");
						else {
							mag = mag + 50000;
		 					(gmtdefs.io_header) ? printf("%.1lf", mag) : printf("%8.1lf", mag);
						}
						break;
					case MANM:	
						if (col.manm == -1) mag = NODATA;
						else
						(col_short[col.manm]) ? (mag = record.sdata[col.manm-6])
									: (mag = record.ldata[col.manm]);
						if (mag == NODATA)
							printf ("NaN");
						else {
		 					(gmtdefs.io_header) ? printf("%.1lf", mag) : printf("%8.1lf", mag);
						}
						break;
					case MFILT:	
						if (col.mflt == -1) mag = NODATA;
						else
						(col_short[col.mflt]) ? (mag = record.sdata[col.mflt-6])
									: (mag = record.ldata[col.mflt]);
						if (mag == NODATA)
							printf ("NaN");
						else {
		 					(gmtdefs.io_header) ? printf("%.1lf", mag) : printf("%8.1lf", mag);
						}
						break;
					case TWTT:	
						if (col.twtt == -1) top = NODATA;
						else
						(col_short[col.twtt]) ? (top = record.sdata[col.twtt-6])
									: (top = record.ldata[col.twtt]);
						if (top == NODATA)
							printf ("NaN");
						else {
							(gmtdefs.io_header) ? printf("%.1lf", top) : printf("%8.1lf", top);
						}
						break;
					case TCORR:	
						if (col.tcor == -1) top = NODATA;
						else
						(col_short[col.tcor]) ? (top = record.sdata[col.tcor-6])
									: (top = record.ldata[col.tcor]);
						if (top == NODATA)
							printf ("NaN");
						else {
							(gmtdefs.io_header) ? printf("%.1lf", top) : printf("%8.1lf", top);
						}
						break;
					case UL0:
						us_save = record.ldata[0];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case UL1:
						us_save = record.ldata[1];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case UL2:
						us_save = record.ldata[2];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case UL3:
						us_save = record.ldata[3];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case UL4:
						us_save = record.ldata[4];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case UL5:
						us_save = record.ldata[5];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case US0:
						us_save = record.sdata[0];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case US1:
						us_save = record.sdata[1];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case US2:
						us_save = record.sdata[2];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case US3:
						us_save = record.sdata[3];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case US4:
						us_save = record.sdata[4];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case US5:
						us_save = record.sdata[5];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case US6:
						us_save = record.sdata[6];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
					case US7:
						us_save = record.sdata[7];
						(gmtdefs.io_header) ? printf("%.3lf", us_save) : printf("%9.3lf", us_save);
						break;
						
					case DISTANCE:	
						(gmtdefs.io_header) ? printf("%.3lf", dist) : printf("%9.3lf", dist);
						break;
					case HEADING:	
						if (heading == NODATA)
							printf ("NaN");
						else
							(gmtdefs.io_header) ? printf("%.1lf", heading) : printf("%6.1lf", heading);
						break;
					case VELOCITY:	
						if (speed == NODATA)
							printf ("NaN");
						else {
							if (speedknots) speed *= 1.943844492; /* convert m/s to knots */
							(gmtdefs.io_header) ? printf("%.2lf", speed) : printf("%6.2lf", speed);
						}
						break;
					case WEIGHT:	
						(gmtdefs.io_header) ? printf("%lg", weight) : printf("%6.2lf", weight);
						break;
					case CRUISELEG_ID:	
						(gmtdefs.io_header) ? printf("%s", cruise) : printf("%s", cruise);
						break;
				}
				((i+1) < n_cols_out) ? printf("\t") : printf("\n");
			}

		}

		if (splitdata) printf(">\n");

		fclose(fp);
		fclose(hp);
		free ((char *)gmt);
	}
	
	exit (0);
}

