/*--------------------------------------------------------------------
 *    The GMTPLUS-system:	@(#)gmtplustrack.c	1.2	1/11/96
 *
 *  Based on...
 *    The GMT-system:	@(#)gmttrack.c	2.7  7/22/93
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * gmttrack will read *.gmt-files and and write PostScript code
 * that will create a navigation plot on a basemap using the
 * projection specified by the user. WESN must be specified on the 
 * command line along with other options for scaling, anotation etc.
 *
 * Author:	Paul Wessel
 * Date:	10-MAR-1988
 * Revised:	22-NOV-1988	Color Version
 *		13-JUN-1991	v2.0
 *
 * 01/11/94 jcm GMT_Plus
 * 05/19/94 Ed Stickgold - Revised -A option
 *
 * GMTPlus version 03/10/95 jcm
 * Modifications made for gmtplus database system.
 *
 * 07/25/01 jjb - GMT3.4 changes, many gmt_ to GMT_ type changes for extern functions,
 *              many changes for "rgb", (colors are now an array)
 *              added colors "black" and "white",
 *              also gmt_ppu[gmtdefs.measure_unit] changed to ps.points_pr_unit
 *                   where "ps" is an external struct from GMT3.4
 *
 * 08/18/04 Ed Stickgold - fixes for GMT 3.4.3 changes, also replaced distance calculation
 *              routine with routine used in gmtpluslist to eliminate minor varation
 *              between the two programs
 *
 * 10/26/04 Ed Stickgold - added call to GMT_put_history to write .gmtcommands file
 * 
 * Plotting etc. algorithm and routines essentially unchanged from
 * gmttrack.c
 *
 */

#include <stdio.h> 
#include <gmt.h>
#include <pslib.h>
#include "gmtplus_mgg.h"

/* ES 8/17/04 - GMT changed way that units are handled */
/* added ps structure declaration from pslib_inc.h */

/* Global structure used internally by pslib */

extern struct {
	FILE *fp;		/* PS output file pointer. NULL = stdout	*/
	int bb[4];		/* Boundingbox arguments			*/
	int font_no;		/* Current font number				*/
	int linewidth;		/* Current pen thickness			*/
	int rgb[3];		/* Current paint				*/
	BOOLEAN landscape;	/* TRUE = Landscape, FALSE = Portrait			*/
	int ix, iy;		/* Absolute coordinates of last point		*/
	int npath;		/* Length of current unstroked path		*/
	char bw_format[8];	/* Format used for grayshade value		*/
	char rgb_format[24];	/* Same, for color triplets			*/
	double xscl, yscl;	/* Global scale values [1.0]			*/
	double xoff, yoff;	/* Origin offset [1/1]				*/
	double scale;		/* Must be set through plotinit();		*/
	int p_width;		/* Paper width in points, set in plotinit();	*/
	int p_height;		/* Paper height in points, set in plotinit();	*/
	double points_pr_unit;	/* # of points pr measure unit (e.g., 72/inch	*/
	int v1_path_length_limit;	/* Implementation limit on path lengths	in Level 1 PostScript	*/
	int max_path_length;	/* Maximum path length encountered during execution	*/
	int clip_path_length;	/* Current length of clip path			*/
	int hex_image;		/* TRUE writes images in hex, FALSE uses binary	*/
	int absolute;		/* TRUE will reset origin, FALSE means relative position	*/
	int eps_format;		/* TRUE makes EPS file, FALSE means PS file	*/
} ps;


#define DST 0.25
#define ANSIZE 0.125
/* KMPRDEG and RAPRDEG are from gmtpluslist.c and should be left
   unaltered to coordinate gmtplustrack output with gmtpluslist output
*/
#define KMPRDEG 111.1949e-6
#define RADPRDEG 0.01745329e-6

double heading();
struct GMT_PLUS_REC *record;
struct COL_N col;
int *distance;
struct ANOT {
	int anot_int_dist;
	int anot_int_time;
	int tick_int_dist;
	int tick_int_time;
	int anotleg_int_dist;
} info;

struct GMT_PEN pen;
int	anotate_year = FALSE;




int get_anotinfo (char *args, struct ANOT *info)
{
	int i1, i2, error = FALSE;
	char flag1, flag2, type;
	double value;
	
	info->anot_int_dist = info->tick_int_dist = 0;
	info->anot_int_time = info->tick_int_time = 0;

	i1 = 0;
	while (args[i1]) {
		flag1 = 'a';
		if (isalpha (args[i1])) {
			flag1 = args[i1];
			if (isupper((int)flag1)) flag1 = tolower ((int)flag1);
			i1++;
		}
		i2 = i1;
		while (args[i2] && strchr ("athHkmM", (int)args[i2]) == NULL) i2++;
		value = atof (&args[i1]);
		flag2 = args[i2];
		/*if (isupper((int)flag2)) flag2 = tolower ((int)flag2);*/
		if (flag2 == 'h') {		/* Hours */
			value *= 3600;
			type = 't';
		}
		else if (flag2 == 'H') {	/* Hours and year */
			anotate_year = TRUE;
			value *= 3600;
			type = 't';
		}
		else if (flag2 == 'k') {	/* kilometers */
			value *= 1000;
			type = 'd';
		}
		else if (flag2 == 'm') {	/* Minutes */
			value *= 60;
			type = 't';
		}
		else if (flag2 == 'M') {	/* Minutes and year */
			anotate_year = TRUE;
			value *= 60;
			type = 't';
		}
		else				/* Default is seconds */
			type = 't';
		i2++;
		if (flag1 == 'a') {	/* Anotation interval */
			if (type == 'd')	/* Distance */
				info->anot_int_dist = value;
			else
				info->anot_int_time = value;
		}
		else {			/* Tickmark interval */
			if (type == 'd')	/* Distance */
				info->tick_int_dist = value;
			else
				info->tick_int_time = value;
		}
		i1 = i2;
	}
	if (info->anot_int_dist <= 0 && info->tick_int_dist <= 0 && info->anot_int_time <= 0 && info->tick_int_time <= 0)
		error = TRUE;
	if (info->anot_int_dist <= 0)
		info->anot_int_dist = info->tick_int_dist;
	else if (info->tick_int_dist <= 0)
		info->tick_int_dist = info->anot_int_dist;
	if (info->anot_int_time <= 0)
		info->anot_int_time = info->tick_int_time;
	else if (info->tick_int_time <= 0)
		info->tick_int_time = info->anot_int_time;
	return (error);
}

int gmt_outside (int lon, int lat, int w, int e, int s, int n)
{
	if (lat < s || lat > n) return (TRUE);
	lon -= 360000000;
	while (lon < w) lon += 360000000;
	if (lon > e) return (TRUE);
	return (FALSE);
}

int anot_legname (double x, double y, int rec, int nrecs,
		  char *text, int size, int w, int e, int s ,int n, int font)
{
	int just;
	double angle;
	
	if (rec == 0) {
		angle = 0.0;
		just = 6;
	}
	else {
		angle = heading (rec, nrecs);
		if (record[rec-1].ldata[col.lat] < s)
			just = (angle >= 0.0) ? 1 : 3;
		else if (record[rec-1].ldata[col.lat] > n)
			just = (angle >= 0.0) ? 11 : 9;
		else if (record[rec-1].ldata[col.lon] < w)
			just = (angle >= 0.0) ? 9 : 1;
		else
			just = (angle >= 0.0) ? 3 : 11;
	}
	ps_setfont (font);
	ps_setpaint (gmtdefs.basemap_frame_rgb);
	ps_text (x, y, size, text, angle, just, 0);
	ps_setpaint (pen.rgb);
	ps_plot (x, y, 3);
	ps_setfont (7);
}

int getmore_anotinfo(char *params, double *size, struct ANOT *info)
{
	char *ptr;

	if (ptr = strchr(params, 'd')) /* set new distance */
		info->anotleg_int_dist = (atof(ptr+1) * 1000.0);
	if (ptr = strchr(params, 's')) /* set new typesize */
		*size = atof(ptr+1) / ps.points_pr_unit ;
	/* fix invalid arguments */
	if (*size <= 0.0) *size = ANSIZE;
	if (info->anotleg_int_dist < 0) info->anotleg_int_dist = 0; /* 0 means no periodic track name annotation */
	return 0;
}



int
main (int argc, char *argv[])
{
	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, lat, lon, angle;
	double anotsize = ANSIZE, factor = 0.001;
	double dx, dy, x, y, x0, y0; 
	double	lon_previous, lat_previous, x_previous, y_previous;
	double	xw, ys, xe, yn, relative_scale_factor;
	double	start_dist = 0.0, stop_dist = 1.0E100;
	int i, iw, ie, is, in, anot_dist, tick_dist, anot_time, tick_time;
	int anotleg_dist;
	int nfiles = 0, n_records, leg_year, rec, pen_type = 0;
	int error = FALSE, first, last = FALSE, anot_name = FALSE, both = FALSE;
	int  anot_tick = FALSE, draw_tick = FALSE, draw_legname = FALSE;
	int OUTSIDERANGE = FALSE, year_error = FALSE;
	int start_time = 0, stop_time = 2000000000;
	int year1, mon1, day1 = 0, hour1, min1, time, year2, mon2, day2 = 0, hour2, min2;
	int this_julian, last_julian, year, month, day, hour, minute, second;
	char agency[10] = "", cruise[80] = "", gmtfile[100], gmtheader[100], label[50];
	char* anot_params;
	int last_lon, last_lat, dlon;
	double dist, ds;
	FILE *fp,*hp;
	struct GMT *gmt;
	
	int white[3] = {255, 255, 255}; /* -jjb 13 JUN 01: should be CONST... */
        int black[3] = {0, 0, 0};       /* but that makes a mockery of its useage. */

	GMT_init_pen (&pen, 1);
	
	GMT_begin (argc, argv);

	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch(argv[i][1]) {
				case 'B':
				case 'P':
				case 'O':
				case 'K':
				case 'R':
				case 'J':
				case 'U':
				case 'V':
				case 'X':
				case 'x':
				case 'Y':
				case 'y':
				case '#':
				error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
					break;
				case 'F':
					sscanf (&argv[i][2], "%d/%d/%d", &gmtdefs.basemap_frame_rgb[0],
						&gmtdefs.basemap_frame_rgb[1], &gmtdefs.basemap_frame_rgb[2]);
					break;
				case 'W':
					GMT_getpen (&argv[i][2], &pen);
					break;
				case 'A':
					anot_name = TRUE;
					anot_params = argv[i]+2;
					/* initialize annotation size and info after calling ps_plotinit */
					break;
				case 'M':
					error += get_anotinfo (&argv[i][2], &info);
					break;
				case 'D':		/* Assign start/stop times for sub-section */
					if (argv[i][2] == 'a') {	/* Start date */
						sscanf(&argv[i][3], "%d/%d/%d/%d:%d",
							&year1, &mon1, &day1, &hour1, &min1);
						if (year1 < 1900) year_error = TRUE;
					}
					else if (argv[i][2] == 'b')	 {	/* Stop date */
						sscanf(&argv[i][3], "%d/%d/%d/%d:%d",
							&year2, &mon2, &day2, &hour2, &min2);
						if (year2 < 1900) year_error = TRUE;

					}
					else
						error = TRUE;
					break;
				case 'S':		/* Assign start/stop position for sub-section */
					if (argv[i][2] == 'a')	/* Start position */
						start_dist = atof(&argv[i][3]);
					else if (argv[i][2] == 'b')	/* Stop position */
						stop_dist = atof(&argv[i][3]);
					else
						error = TRUE;
					break;
				default:
					error = TRUE;
					break;
			}
		}
		else
			nfiles++;
	}
	if (nfiles == 0) error = TRUE;
	if ((pen.rgb[0] < 0 || pen.rgb[0] > 255) || (pen.rgb[1] < 0 || pen.rgb[1] > 255) ||
		 (pen.rgb[2] < 0 || pen.rgb[2] > 255)) error = TRUE;

	if (year_error) {

		fprintf(stderr,"\n\tError: Enter full year in -D option, for example, 1967 instead of just 67\n\n");
		exit(-1);
	}

		 
	if (error) {
		fprintf(stderr,"usage: gmtplustrack leg(s) -R<west>/<east>/<south>/<north> -J<params> [-B<tickinfo>]\n");
		fprintf(stderr,"	[-P] [-F<r>/<g>/<b>] [-W<pen>] [-A[s<size>][d<dist>]] [-M<trackticks>] [-U]\n");
		fprintf(stderr,"	[-#<ncopies>] [-V] [-X<x_shift>] [-Y<y_shift>]\n\n");
		fprintf (stderr, "	-A will Annotate legs when they enter the grid. <size> is optional character\n");
		fprintf (stderr, "	   size in points [9]. If a distance is given, each track will be annotated with\n");
		fprintf (stderr, "	   its leg name every <dist> kilometers.\n");
		GMT_explain_option ('B');
		fprintf (stderr, "	-F Set color used for Frame and anotation [%d/%d/%d]\n",
			gmtdefs.basemap_frame_rgb[0], gmtdefs.basemap_frame_rgb[1], gmtdefs.basemap_frame_rgb[2]);
		GMT_explain_option ('J');
		GMT_explain_option ('K');
		GMT_explain_option ('O');
		GMT_explain_option ('P');
		GMT_explain_option ('R');
		GMT_explain_option ('U');
		GMT_explain_option ('V');
		GMT_explain_option ('X');
		GMT_explain_option ('#');
		GMT_explain_option ('.');
		fprintf (stderr, "	-M to put time/distance Marks on the track. Flags 'a' and 't' specify (a)nnotate\n");
		fprintf (stderr, "	   or (t)ick marks, and have the form a<value><flags> where <flags> are the\n");
		fprintf (stderr, "	   case-sensitive h,H,m,M,k. E.g. -Ma6ht50k (a)nnotates the track every 6 (h)ours\n");
		fprintf (stderr, "	   and ticks it every 50 (k)ilometers, -Mt15m (t)icks (and automatically annotates)\n");
		fprintf (stderr, "	   the track every 15 (m)inutes. 'H' and 'M' are the same as 'h' and 'm' but also\n");
		fprintf (stderr, "	   print the cruise year as part of the date labels.\n");
		fprintf(stderr,"	-Da<date> only plot track from date (given as yyyy/mm/dd/hh:mm)\n");
		fprintf(stderr,"	-Db<date> only plot track up to date (given as yyyy/mm/dd/hh:mm)\n");
		fprintf(stderr,"	-Sa<dist> only plot track from along-track distance dist (in km)\n");
		fprintf(stderr,"	-Sb<dist> only plot track up to along-track distance dist (in km)\n");
		exit (-1);
	}
	

	GMT_put_history (argc,argv);	/* Update .gmtcommands */
	gmtpath_init();

	if (east < west) west -= 360.0;
		
	GMT_map_setup (west, east, south, north);
	

	/* GMT_LEVEL defined in gmt.h - (/sw/include with fink) */
#ifdef GMT_LEVEL
	/*
	ps_plotinit (NULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
		gmtdefs.dpi, gmtdefs.measure_unit, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
	*/
	fprintf (stderr, "FIX ERROR!!!! SORRY - need to port call to ps_plotinit to GMT 4\n");
#else /* GMT 3 I hope! */
	ps_plotinit (NULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
		gmtdefs.dpi, gmtdefs.measure_unit, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
#endif

	if (anot_name) getmore_anotinfo(anot_params, &anotsize, &info); /* this must run after ps_plotinit */
		
	GMT_echo_command (argc, argv);
	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
	
	ps_setpaint (gmtdefs.basemap_frame_rgb);
	
	ps_setline (pen.width);
	if (pen.texture) ps_setdash (pen.texture, pen.offset);
	distance = (int *) GMT_memory (CNULL, 1, sizeof (int), "gmttrack");
	record = (struct GMT_PLUS_REC *) GMT_memory (CNULL, 1, sizeof (struct GMT_PLUS_REC), "gmttrack");

	ps_setfont (7);	/* Times Italic */
	GMT_map_clip_on (GMT_no_rgb, 3);
	iw = west * 1.0e6;	ie = east * 1.0e6;	is = south * 1.0e6;	in = north * 1.0e6;
	both = (info.anot_int_time && info.anot_int_dist);
	ps_setpaint (pen.rgb);
	
	/* now find maximum plot dimension on paper and set the along-track 
	   gap specifier "relative_scale_factor" to be 1/20th of this*/

	GMT_geo_to_xy (west, south, &xw, &ys);
	GMT_geo_to_xy (east, north, &xe, &yn);

	if (xe == xw) relative_scale_factor = 0.05*xe;
	else if ((xe - xw) > (yn - ys)) relative_scale_factor = 0.05*(xe - xw);
	else relative_scale_factor = 0.05*(yn - ys);

	for (i = 1; i < argc; i++) {	/* Loop over arguments and open/plot the gmtfiles */
		if (argv[i][0] == '-') continue;
  		if (gmtpath_func(gmtfile,gmtheader,argv[i])) {
   			fprintf(stderr,"gmtplustrack : Cannot find leg %s\n", argv[i]);
     			continue;
  		}
		if ((fp = fopen(gmtfile, "r")) == NULL) {
			fprintf(stderr, "gmtplustrack: Could not open datafile %s\n", gmtfile);
			continue;
		}
		if ((hp = fopen(gmtheader, "r")) == NULL) {
			fprintf(stderr, "gmtplustrack: Could not open headerfile %s\n", gmtheader);
			fclose(fp);
			continue;
		}
		
		read_gmt_master_index();
		/* read header file */
		read_gmt_header(hp, &col, &leg_year, &n_records, agency, cruise); 
		/* read first record in data file */
		read_first_gmt_datarec(fp, &col);
		/* begin reading data */
		
		if (leg_year < 1900) leg_year += 1900;

		distance = (int *) GMT_memory ((char *)distance, n_records, sizeof (int), "gmttrack");
		record = (struct GMT_PLUS_REC *) GMT_memory ((char *)record, n_records, sizeof (struct GMT_PLUS_REC), "gmttrack");
		
		/* Read cruise data */

		for (rec = 0; rec < n_records; rec++) {
			/*if (fread((char *)(&record[rec]), 40, 1, fp) != 1) {*/
			/* 08/31/95 jcm Block out flagged data */
			if (read_data_record(&record[rec], &col, fp) == -1) {
				fprintf(stderr,"gmtplustrack: Error reading data record no %d\n",rec);
				exit(-1);
			}

			/* Distance algorithm taken from gmtpluslist so that gmtplustrack
			   output remains in sync with gmtpluslist output */
		
			/* Compute accumulated distance along track (Flat Earth) */

			if (rec == 0) {
				last_lon = record[rec].ldata[col.lon];
				last_lat = record[rec].ldata[col.lat];
				ds = dist = 0.0;
			}
			else {
				dlon = record[rec].ldata[col.lon] - last_lon;
				if (abs (dlon) > 180000000) dlon = copysign ((double) (360000000 - abs (dlon)), (double)dlon);
				dx = (double) dlon * cos (0.5*RADPRDEG*(double)(record[rec].ldata[col.lat]+last_lat)) * KMPRDEG;
				dy = (double) (record[rec].ldata[col.lat] - last_lat) * KMPRDEG;
				ds = hypot (dx, dy);
				last_lon = record[rec].ldata[col.lon];
				last_lat = record[rec].ldata[col.lat];
			}
			dist += ds;
			distance[rec] = rint(dist * 1.0e3);				
		}
		fclose (fp);
		fclose(hp);
		
		gmt = gmt_init(leg_year);	/* Initialize gmt_structure */
		last_julian = -1;
		if (gmtdefs.verbose) fprintf (stderr, "gmtplustrack: Plotting %s\n", agency);
		printf ("%% Tracking %s\n", agency);
		

		/* First draw the track line, clip segments outside the area */
		
		first = TRUE;
		last = FALSE;
		for (rec = 0; rec < n_records; rec++) {

			if (gmt_outside (record[rec].ldata[col.lon], record[rec].ldata[col.lat], iw, ie, is, in)) {
				if (last && rec < (n_records-1)) {
					lon = record[rec].ldata[col.lon] * 1.0e-6;
					lat = record[rec].ldata[col.lat] * 1.0e-6;
					GMT_geo_to_xy (lon, lat, &x, &y);
					ps_plot (x, y, 3);	/* lift pen if pt is outside -R region */
					last = FALSE;
				}
				first = TRUE;
				continue;
			}
			lon = record[rec].ldata[col.lon] * 1.0e-6;
			lat = record[rec].ldata[col.lat] * 1.0e-6;
			GMT_geo_to_xy (lon, lat, &x, &y);

			if (first) {
				if (rec > 0) {
					lon = record[rec].ldata[col.lon] * 1.0e-6;
					lat = record[rec].ldata[col.lat] * 1.0e-6;
					GMT_geo_to_xy (lon, lat, &x0, &y0);
					ps_plot (x0, y0, 3);
				}
				else
					ps_plot (x, y, 3);
				if (anot_name) anot_legname (x, y, rec, n_records, argv[i], (int) rint (ps.points_pr_unit * anotsize), iw ,ie, is, in, 4);
				first = FALSE;
				if (info.anot_int_dist > 0) anot_dist = (distance[rec] / info.anot_int_dist + 1) * info.anot_int_dist;
				if (info.tick_int_dist > 0) tick_dist = (distance[rec] / info.tick_int_dist + 1) * info.tick_int_dist;
				if (info.anot_int_time > 0) anot_time = (record[rec].ldata[col.tsec] / info.anot_int_time + 1) * info.anot_int_time;
				if (info.tick_int_time > 0) tick_time = (record[rec].ldata[col.tsec] / info.tick_int_time + 1) * info.tick_int_time;
				if (info.anotleg_int_dist > 0) anotleg_dist = (distance[rec] / info.anotleg_int_dist + 1) * info.anotleg_int_dist;
			}

			/* next line stops the program from joining the ends of track segments 
			across the chosen central meridian, and tests for any gaps along-track: 
			if the longitude or latitude difference between the current point and 
			the preceding one transforms onto a paper position greater than 1/20th 
			the maximum paper plot dimension then stroke, lift the pen and move to 
			this new point's position ("ps_plot (x, y, 3)"). Otherwise, just draw 
			a line to the new point ("ps_plot (x, y, 2)"). 

			Calculate position on paper of the previous point by calling 
			up record[rec-1] values. For the first point, rec = 0 which may 
			cause problems because record[-1] isn't defined. Overcome this 
			by testing for "first" being FALSE - only on rec = 0 is it TRUE, 
			for all subsequent points it's FALSE. In the lines above, when 
			first is TRUE, the pen is moved to the first point's position 
			using "ps_plot (x, y, 3)" so there's no need to have an extra 
			case below to test tell the program what to do if we are on the 
			first point, that is, only do the following lines when first is FALSE.
			Modified by Andrew Goodwillie, SIO, 1/11/96 */

			if (!first) {

				lon_previous = record[rec-1].ldata[col.lon] * 1.0e-6;
				lat_previous = record[rec-1].ldata[col.lat] * 1.0e-6;
				GMT_geo_to_xy (lon_previous, lat_previous, &x_previous, &y_previous);

				if (day1 > 0) gmt_time(&start_time, year1, mon1, day1, hour1, min1, 0, gmt);
				if (day2 > 0) gmt_time(&stop_time,  year2, mon2, day2, hour2, min2, 0, gmt);


				if ((fabs(x - x_previous) > relative_scale_factor) || (fabs(y - y_previous) > relative_scale_factor)) ps_plot (x, y, 3);

				else if (((float)(distance[rec]))/1000.0 < start_dist || ((float)(distance[rec]))/1000.0 > stop_dist) {

					ps_plot (x, y, 3); 
					OUTSIDERANGE = TRUE;
				}
				else if (record[rec].ldata[col.tsec] < start_time || record[rec].ldata[col.tsec] > stop_time) {

					ps_plot (x, y, 3); 
					OUTSIDERANGE = TRUE;
				}
				else ps_plot (x, y, 2);
			}

			last = TRUE;
			
			/* See if we need to anotate/tick the trackline for time/km/legname marks */
			
			if (info.anot_int_time && (record[rec].ldata[col.tsec] >= anot_time)) {
				anot_time += info.anot_int_time;
				anot_tick = 1;
			}
			if (info.anot_int_dist && (distance[rec] >= anot_dist)) {
				anot_dist += info.anot_int_dist;
				anot_tick += 2;
			}
			if (info.tick_int_time && (record[rec].ldata[col.tsec] >= tick_time)) {
				tick_time += info.tick_int_time;
				draw_tick = 1;
			}
			if (info.tick_int_dist && (distance[rec] >= tick_dist)) {
				tick_dist += info.tick_int_dist;
				draw_tick += 2;
			}

			if (info.anotleg_int_dist && (distance[rec] >= anotleg_dist))
			{
				anotleg_dist += info.anotleg_int_dist;
				draw_legname = TRUE;
			}

			if (anot_tick) {
				angle = heading (rec, n_records);
				if (angle < 0.0)
					angle += 90.0;
				else
					angle -= 90.0;
				if (anot_tick & 1) {	/* Time mark */
					this_julian = gmt_date (anot_time - info.anot_int_time,&year,&month,&day,&hour,&minute,&second,gmt);
					printf ("S\n");
					if (this_julian != last_julian) {
						if (anotate_year) sprintf (label, " %02d:%02d %02d/%02d/%d\0", hour, minute, month, day, year);
						else sprintf (label, " %02d:%02d %02d/%02d\0", hour, minute, month, day);
						if (!OUTSIDERANGE) ps_circle (x, y, 0.2*anotsize, black, TRUE);
						ps_setfont (7);
					}
					else {
						sprintf (label, " %02d:%02d\0", hour, minute);
						if (!OUTSIDERANGE) ps_circle (x, y, 0.2*anotsize, white, TRUE);
					}
					ps_setpaint (gmtdefs.basemap_frame_rgb);
					if (!OUTSIDERANGE) ps_text (x, y, (int)rint (ps.points_pr_unit * anotsize), label, angle, 5, 0);
					if (this_julian != last_julian) ps_setfont (6);
					last_julian = this_julian;
				}
				if (anot_tick & 2) {	/* Distance mark */
					printf ("S\n");
					sprintf (label, "%d km  \0", (int)((anot_dist - info.anot_int_dist) * factor));
					if (!OUTSIDERANGE) ps_square (x, y, 0.4*anotsize, gmtdefs.basemap_frame_rgb, FALSE);
					ps_setpaint (gmtdefs.basemap_frame_rgb);
					if (!OUTSIDERANGE) ps_text (x, y, (int) rint (ps.points_pr_unit * anotsize), label, angle, 7, 0);
				}
			}
			if (both && !(anot_tick & 1) && (draw_tick & 1)) {
				printf ("S\n");
				if (!OUTSIDERANGE) ps_circle (x, y, 0.2*anotsize, gmtdefs.basemap_frame_rgb, FALSE);
			}
			if (both && !(anot_tick & 2) && (draw_tick & 2)) {
				printf ("S\n");
				if (!OUTSIDERANGE) ps_square (x, y, 0.4*anotsize, gmtdefs.basemap_frame_rgb, FALSE);
			}
			if (draw_tick) {
				printf ("S\n");
				ps_setpaint (gmtdefs.basemap_frame_rgb);
				if (!OUTSIDERANGE) ps_cross (x, y, 0.4*anotsize);
			}
			if (draw_legname) 
				anot_legname (x, y, rec, n_records, argv[i], (int) rint (ps.points_pr_unit *  anotsize), iw ,ie, is, in, 4);

			if (anot_tick || draw_tick || draw_legname) {
				ps_setpaint (pen.rgb);
				ps_plot (x, y, 3);
				anot_tick = draw_tick = draw_legname = FALSE;
			}

			OUTSIDERANGE = FALSE;
		}
	}
	GMT_map_clip_off();
	if (pen_type > 0) ps_setdash (NULL, 0);
	if (frame_info.plot) {
		ps_setpaint (gmtdefs.basemap_frame_rgb);
		GMT_map_basemap ();
		ps_setpaint (black);
	}
	ps_plotend (gmtdefs.last_page);
	free ((char *)record);
	free ((char *)distance);
	
	GMT_end (argc, argv);
}


double heading (rec, n_records)
int rec, n_records; {
	int i1, i2;
	double angle, lon, lat, x1, x2, y1, y2;
	
	i1 = rec - 25;
	if (i1 < 0) i1 = 0;
	i2 = i1 + 25;
	if (i2 > (n_records-1)) i2 = n_records - 1;
	lon = record[i2].ldata[col.lon] * 1.0e-6;	lat = record[i2].ldata[col.lat] * 1.0e-6;
	GMT_geo_to_xy (lon, lat, &x2, &y2);
	lon = record[i1].ldata[col.lon] * 1.0e-6;	lat = record[i1].ldata[col.lat] * 1.0e-6;
	GMT_geo_to_xy (lon, lat, &x1, &y1);
	angle = atan2 (y2 - y1, x2 - x1) * R2D;
	if (angle > 90.)
		angle -= 180;
	else if (angle < -90.0)
		angle += 180.0;
	return (angle);
}


