/*--------------------------------------------------------------------
 *    The GMTPLUS-system:	@(#)igrfgmtplus.c	1.2	9/12/95
 *
 *  Based on...
 *    The GMT-system:	@(#)mgd77togmt.c	2.6  7/23/93
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * igrfgmtplus takes an existing gmtplus file and recalculates
 * mag anomaly based on total magnetic field.  A valus in the
 * mag anomaly column will be overwritten if there is a valid
 * value in the total magnetic field column.  If the magtot column
 * does not have a valid value, the value in the mananom column
 * will remain unchanged.
 *
 * This program will create new gmtplus files, and rename the
 * original files with a *.old extension.  Please allow sufficient
 * space for these additional files.  Additionally, pedigree lines
 * will be added to the header file to indicate this process has
 * been run, when, and by whom.
 *
 * The methodology used to calculate the mag filed from which the
 * anomaly is extracted is that of the SIO mag field routine fieldx_,
 * written and maintained by Stu Smith.  This program is linked to
 * that library so a recompile is recommended if the mag library
 * is updated.  Watch out for the 5-year expiration on mag values!
 *
 * GMTPlus 03/10/95 jcm
 *
 * 8/19/04 Ed Stickgold - minor fixes for GMTPlus, also revised for
 * call for new fortran magnetic field routine.
 */
#include <time.h>
#include <unistd.h> /* gethostname() */
#include <gmt.h>
#include "gmtplus_mgg.h"

#define NODATA -32000


struct GMT *gmt;
struct GMT_PLUS_REC *record;
struct COL_N col;
int T_INC = 60, fake_running_time = 0;


void
decode_record (int lat,int lon,int tsec,int leg_year,short int mtot,short int *manom)
/*
int lat, lon, tsec;
int leg_year;
short int mtot, *manom;
*/
{
   float ylat, ylon, alt, date;
   float fieldx;   /* north    component of the magnetic force in nT - unused; */
   float fieldy;   /* east     component of the magnetic force in nT - unused; */
   float fieldz;   /* downward component of the magnetic force in nT - unused; */
   float fieldtot; /* total magnetic force in nT;*/

   ylat = lat / 1000000.0;
   ylon = lon / 1000000.0;
   alt  = 0.0;

   date = leg_year;

   if (mtot != NODATA) {
      igrf2000_(&ylon,&ylat,&alt,&date,&fieldx,&fieldy,&fieldz,&fieldtot);
      *manom = (int)((mtot+50000) - fieldtot);
   }

}


/* char *memory(); */
int 
main (int argc, char *argv[])
{
  int i, j, rec, n_records, n_alloc = GMT_CHUNK, leg_year;
  int t_flag = FALSE, error = FALSE, header = FALSE;
  char file[80], cruise[80] = "", agency[10] = "", legfname[80], line[512];
    char gmtfile[100], gmtheader[100];
	FILE *fp = NULL, *fpo, *hpo, *hp;
    int column;
    int verbose = 0;
    int nfiles = 0;
    char comment[80];
    long int    right_now;
    char    date[25], user[128], host[128];


    gmtpath_init();
    read_gmt_master_index();

    for (j=1; j<argc; j++) 
		if (!strcmp(argv[j], "-v") || !strcmp(argv[j], "-V")) verbose = TRUE;

    for (j=1; j<argc; j++) {  /* loop over gmt files */

       if (argv[j][0] == '-') continue;
        if (gmtpath_func (gmtfile, gmtheader, argv[j])) {
            fprintf(stderr,"igrfgmtplus : Cannot find leg %s\n", argv[j]);
                continue;
        }
        if ((hp = fopen (gmtheader, "r")) == NULL) {
            fprintf(stderr,"igrfgmtplus: Could not open header file %s\n", gmtheader);
            continue;
        }
        if ((fp = fopen (gmtfile, "r")) == NULL) {
            fprintf(stderr,"igrfgmtplus: Could not open data file %s\n", gmtfile);
            fclose(hp);
            continue;
        }
        nfiles++;

        if (verbose) fprintf (stderr, "Now reading leg %s\n", argv[j]);

        read_gmt_header(hp, &col, &leg_year, &n_records, agency, cruise);

		if (col.mtot == -1) {
			if (verbose) 
				fprintf(stderr, "Leg %s does not contain total magnetic field data; Skipping.\n", cruise);
			fclose(hp);
			fclose(fp);
			continue;
		}
        if (leg_year < 1900) leg_year += 1900;
        read_first_gmt_datarec(fp, &col);
        if ((col.manm == -1) && (col.mtot != -1)) col.manm = 10;

        gmt = gmt_init (leg_year);  /* Initialize gmt_structure */


	record = (struct GMT_PLUS_REC *) GMT_memory ((char *)NULL, n_records, sizeof (struct GMT_PLUS_REC), "igrfgmtplus");

	rec = 0;
    for (rec=0; rec<n_records; rec++) {

       if (fread((char *)(&record[rec]), 40, 1, fp) != 1) {
          fprintf(stderr,"igrfgmtplus: Error reading data record no. %d\n",rec);
          exit(-1);
       }

	   decode_record (record[rec].ldata[col.lat],
           record[rec].ldata[col.lon],
           record[rec].ldata[col.tsec],
           leg_year,
           record[rec].sdata[col.mtot-6],
           &record[rec].sdata[col.manm-6]);

	}
    fclose(hp);
    fclose(fp);
    sprintf(file,"%s.old\0",gmtheader);
    if (rename(gmtheader,file) != 0) {
       fprintf(stderr,"File rename failed: %s\n",file);
       exit(-1);
    }
    sprintf(file,"%s.old\0",gmtfile);
    if (rename(gmtfile,file) != 0) {
       fprintf(stderr,"File rename failed: %s\n",file);
       exit(-1);
    }
	free ((char *)gmt);
	
	if ((hpo = fopen (gmtheader, "w")) == NULL) {
		fprintf (stderr, "Could not create header file %s\n", file);
		exit (-1);
	}
	if ((fpo = fopen (gmtfile, "w")) == NULL) {
		fprintf (stderr, "Could not create data file %s\n", file);
		exit (-1);
	}

    /* assume no corrected data; fix if one was actually set */
    column = col.manm;
    col.manm = -1;
    for (rec=0; rec < n_records; rec++) {
       if (record[rec].sdata[column-6] != NODATA) {
          col.manm = column;
          break;
       }
    }

	write_gmt_header(hpo, &col, leg_year, n_records, agency, cruise, gmtfile);
	write_first_gmt_datarec(fpo, &col);

	for (rec = 0; rec < n_records; rec++) {
		if (fwrite((char *)(&record[rec]), 40, 1, fpo) != 1) {
			fprintf(stderr,"igrfgmtplus: Error writing data record no %d\n",rec);
			exit(-1);
		}
	}

	fclose(fpo);
	fclose(hpo);

	/* add pedigree comments */ 
    sprintf(file,"%s.old\0",gmtheader);
    hp = fopen(file,"r");
    hpo = fopen(gmtheader,"a");
    transfer_header_comments(hp, hpo);

    add_pedigree(hpo,"#FT: gmtplus by igrfgmtplus");

    strncpy(comment,"\0",80);
    sprintf(comment,"#FN: %s",gmtheader);
    add_pedigree(hpo,comment);

    strncpy(date,"\0",24);
    right_now = time((long *)0);
    strcpy(date,ctime(&right_now));
    date[24] = 0; /* punt the nl */
    strcpy(user,getenv("USER"));
    gethostname(host,128);

    strncpy(comment,"\0",80);
    sprintf(comment,"#FD:%s",date);
    add_pedigree(hpo, comment);

    strncpy(comment,"\0",80);
    sprintf(comment,"#HU:%s %s",host,user);
    add_pedigree(hpo, comment);
 
    add_pedigree(hpo,"#*end");
 
    fclose(hp);
    fclose(hpo);
    
    free((char *)gmt);	
	free ((char *)record);

    }

    if (nfiles == 0) {
        fprintf(stderr, "usage: igrfgmtplus leg(s)\n");
        exit(EXIT_FAILURE);
    }

	
    /*exit(0);*/
    return(EXIT_SUCCESS);
}


