/*--------------------------------------------------------------------
 *    The GMTPLUS-system:	@(#)mgd77togmtplus.c	1.2	9/12/95
 *
 *  Based on...
 *    The GMT-system:	@(#)mgd77togmt.c	2.6  7/23/93
 *
 *    Copyright (c) 1991 by P. Wessel and W. H. F. Smith
 *    See README file for copying and redistribution conditions.
 *--------------------------------------------------------------------*/
/*
 * mgd77togmt reads a mgd77 tape file from NGDC and creates
 * a gmt file directly.  If no time is provided, a fake time is
 * issued based on a year and a time increment. Use 1900 for
 * these cruises and estimate the time_inc based on the distance
 * between 2 nav points and an assumed velocity.
 *
 * Author:	Paul Wessel
 * Date:	9-SEP-1989
 *
 * Modified by WHFS 21 July 1993:
 *	Fixed usage message to be consistent with actual run for -H.
 *	Added carter table correction routines for depth.  Now uses
 *	twt if present, and then if not uses corr depth if present.
 *
 * GMTPlus version 03/10/95 jcm
 * Modifications made for gmtplus database system.
 *
 * Release 1.1 (Beta)
 * This program *does not* do Carter corrections!
 * If there is a value in the corrected meters field of the mgd77 record,
 * that value is transferred to the gmtplus file, but if the meters field
 * is empty, the gmtplus field gets the NODATA value.  Thus, the gmtplus
 * file is a copy of the mgd77file, no extra calculations performed.
 *
 * See cartergmtplus for corrected meters calculations.
 *
 */
#include "gmt.h"
#include "gmtplus_mgg.h"

#define NODATA -32000
#define TRUE 1
#define FALSE 0
#define N_BINS 64800
#define N_CARTER_ZONES 85
#define N_CARTER_OFFSETS 86
#define N_CARTER_CORRECTIONS 5812
#define MAX_VALID_TWT   16000   /* Two-way time (msec) to 12000 nominal m  */

int     	debug = 0;
int     	carter_setup();
int     	get_bin();
int     	get_carter_zone();
int     	get_depth_from_twt();
int     	get_twt_from_depth();

short int	carter_zone[N_BINS];
short int       carter_offset[N_CARTER_OFFSETS];
short int       carter_correction[N_CARTER_CORRECTIONS];
struct 		GMT *gmt;
struct 		GMT_PLUS_REC *record;
struct 		COL_N col;
int 		T_INC = 60, fake_running_time = 0;
int		carter_not_initialized = 1;
int 		leg_year = 0;
int 		year_set = FALSE;
char 		path[80];

/****************************************************************/
/***** SUBS *****************************************************/
/****************************************************************/
/****************************************************************/

int decode_record (string, tflag, time, lon, lat, graw, faa,
                   mag, manom, itwt, itcor)
char *string;
int tflag;
int *time, *lon, *lat, *graw;
short int *faa, *mag, *manom, *itwt, *itcor;
{
	int year, month, day, hour, min, sec, l_mag, l_top, test, bin, zone, l_twt;
	int year_short;
	short int	twt;
	double tz, fmin;
	char s_tz[5], s_yr[3], s_mo[3], s_dy[3], s_hr[3], s_mi[6], s_lat[8], s_lon[9];
	char s_faa[10], s_mag[10], s_top[7], s_top_twt[7];
	/* 03/18/94 jcm kludge to keep tcor in */

	/* First decode Time Zone */

	strncpy (s_tz, &string[10], 4);	s_tz[4] = 0;
	tz = atof (s_tz) * 0.01;
	if (string[9] == '-') tz = -tz;

	/* Then decode Time */

	if (tflag) {	/* No time provided, use fake time */
		*time  = fake_running_time;
		fake_running_time += T_INC;
	}
	else {
		strncpy (s_yr, &string[14], 2);	s_yr[2] = 0;
		year_short = atoi (s_yr);

		/** OK here is a bad programming **/
		/** trick to fix a bug  **/
		if( year_short > 50)
		{year = atoi (s_yr) + 1900; }
		else
		{year = atoi (s_yr) + 2000;}
                /** ********************************/


		if (leg_year == 0) {
			leg_year = year;
			gmt = gmt_init (leg_year);
		}

		strncpy (s_mo, &string[16], 2);	s_mo[2] = 0;
		month = atoi (s_mo);

		strncpy (s_dy, &string[18], 2);	s_dy[2] = 0;
		day = atoi (s_dy);

		strncpy (s_hr, &string[20], 2);	s_hr[2] = 0;
		hour = atoi (s_hr);

        /*##################################
        if(debug){fprintf (stderr, " leg_year = %d  month = %d  day = %d  time1 = %f \n", leg_year,month,day,test_time);     }
        /*##################################*/

		strncpy (s_mi, &string[22], 5);	s_mi[5] = 0;
		fmin = atof (s_mi) * 0.001;
		min = floor (fmin);
		sec = rint (60.0 * (fmin - min));
		test = gmt_time (time, year, month, day, hour, min, sec, gmt);
		if (test < 0) return (1);
		*time += rint (tz * 3600.0);

        /*##################################
        if(debug){fprintf (stderr, " leg_year = %d  month = %d  day = %d  time2 = %d test = %d \n", leg_year,month,day,time,test);     }
        /*##################################*/


	}

	/* Get lat lon */

	strncpy (s_lat, &string[28], 7);	s_lat[7] = 0;
	*lat = 10 * atoi (s_lat);
	if (string[27] == '-') *lat = -(*lat);

	strncpy (s_lon, &string[36], 8);	s_lon[8] = 0;
	*lon = 10 * atoi (s_lon);
	if (string[35] == '-') *lon = -(*lon);
	if ((*lon) < 0) *lon += 360000000;

	/* Get gravity */

	strncpy (s_faa, &string[104], 4);	s_faa[4] = 0;
	*faa = atoi (s_faa);
	if ((*faa) == 9999 || (s_faa[0] == ' ' && s_faa[1] == ' ' && s_faa[2] == ' ' && s_faa[3] == ' '))
		*faa = NODATA;
	else if (string[103] == '-')
		*faa = -(*faa);

	strncpy (s_faa, &string[90], 7);	s_faa[7] = 0;
	*graw = atoi (s_faa);
	if ((*graw) == 9999999 || (s_faa[0] == ' ' && s_faa[1] == ' ' && s_faa[2] == ' ' && s_faa[3] == ' '))
		*graw = NODATA;

	/* Get magnetics */

	strncpy (s_mag, &string[73], 5);	s_mag[5] = 0;
	l_mag = atoi (s_mag);
	if (l_mag == 99999 || (s_mag[0] == ' ' && s_mag[1] == ' ' && s_mag[2] == ' ' && s_mag[3] == ' ' && s_mag[4] == ' '))
		l_mag = NODATA;
	else if (string[72] == '-')
		l_mag = -l_mag;
	if (l_mag != NODATA) l_mag = rint (0.1 * l_mag);
	*manom = l_mag;

	strncpy (s_mag, &string[60], 6);	s_mag[6] = 0;
	l_mag = atoi (s_mag);
	if (l_mag == 999999 || (s_mag[0] == ' ' && s_mag[1] == ' ' && s_mag[2] == ' ' && s_mag[3] == ' ' && s_mag[4] == ' '))
		l_mag = NODATA;
	if (l_mag != NODATA) {
       l_mag = rint (0.1 * l_mag);
	   *mag = l_mag - 50000;
    } else {
       *mag = l_mag;
    }

	/* Get Bathymetry */
    /* 11/15/94 jcm include whichever is found, dont do calcs */
    /* corrected depth */
	strncpy (s_top, &string[51], 6);	s_top[6] = 0;
	l_top = atoi (s_top);
	if (l_top == 999999 || (s_top[0] == ' ' && s_top[1] == ' ' && s_top[2] == ' ' &&
		s_top[3] == ' ' && s_top[4] == ' '&& s_top[5] == ' '))
		l_top = NODATA;
	else
		l_top = -rint (0.1 * l_top);
    *itcor = l_top;

    /* two-way travel time */
	strncpy(s_top_twt, &string[45], 6);	s_top_twt[6] = '\0';
	l_twt = atoi(s_top_twt);
	if (l_twt == 999999 || (s_top_twt[0] == ' ' && s_top_twt[1] == ' ' && s_top_twt[2] == ' ' &&
		s_top_twt[3] == ' ' && s_top_twt[4] == ' '&& s_top_twt[5] == ' ')) {
		*itwt = NODATA;
	}
	else {
	  twt = rint(0.1 * l_twt); /* FIX: make sure that rint is the same as fortran nint */
	  /*twt = nint(0.1 * l_twt);*/
	  *itwt = twt;
        /* 11/15/94 jcm do *not* do conversions yet */
	/* 09 AUG 2001 jjb removed dead code that jcm does *NOT* use */
	}
	return (0);
}
/****************************************************************/
/****************************************************************/



int
main (int argc, char *argv[])
{
	int i, rec, n_records, n_alloc = GMT_CHUNK;
	int t_flag = FALSE, error = FALSE, header = FALSE; char *envdir;

	char file[80], agency[10] = "", legfname[80], line[512];
	FILE *fp = NULL, *fpo, *hpo;
    int column;

	agency[0] = '\0';
        if(debug){fprintf (stderr, "DEBUG MODE ON\n");     }
	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
				case 'Y':
					leg_year = atoi (&argv[i][2]); year_set = TRUE;
					break;
				case 'I':	/* No time in data file, specify time increment between points */
					T_INC = atoi (&argv[i][2]);
					t_flag = TRUE;	/* fake time starts at 1/1/1900 @ 00:00:00 hr */
					break;
				case 'H':	/* File has 16 header records */
					header = TRUE;
					break;
				case 'F':
					strcpy (legfname, &argv[i][2]);
					break;
				case 'A':
					strcpy (agency, &argv[i][2]);
					break;
			        case 'P':	/* Give path to index files */
					strcpy (path, &argv[i][2]);
					break;
				default:
					error = TRUE;
					break;
			}
		}
		else
			fp = fopen (argv[i], "r");
	}

	if (fp == NULL) fp = stdin;

	if (argc == 1 || error) {
		fprintf (stderr, "usage: mgd77togmtplus -Ffilename [-Pcarter_path] [-Yleg_year] [-A<10 char agency name>] [NGDC-file -H -Itime_increment]\n");
		fprintf (stderr, "	-Y sets start year\n");
		fprintf (stderr, "	-H means that the 16 MGD-77 header records are present\n");
		fprintf (stderr, "	-I sets fake timeincrement for files without time information\n");
		fprintf (stderr, "	-F sets gmtfilename prefix (e.g., without path or .gmt)\n");
		fprintf(stderr,"	-A sets optional 10 char info for gmt header.\n");
		fprintf(stderr,"	-P gives directory for the carter tables (default is %s)\n", LIBDIR);

		exit (EXIT_FAILURE);
	}

        if(year_set == FALSE){fprintf(stderr,"\n WARNING: Year has not been set with -Y \n"); }
	/* carter table path priority:
	   1) -P argument
	   2) GMTPLUS_LIBDIR
	   3) default (LIBDIR)
	*/

	if (path[0] == 0) {
		envdir = getenv("GMTPLUS_LIBDIR");
		if (envdir == NULL) {
		  sprintf (path, "%s", LIBDIR);
		} else {
		  sprintf (path, "%s", envdir);
		}
	}
	/* printf("FIXME: looking for carter data in \"%s\"\n", path); */

    /* Set up default fields, to be updated later. */
	read_gmt_master_index();
	col.tsec = 0;
	col.lat = 1;
	col.lon = 2;
	col.graw = 3;
	col.gfaa = 6;
	col.twtt = 7;
	col.tcor = 8;
	col.mtot = 9;
	col.manm = 10;
	col.mflt = -1;

	col.bad_flag = -1;
	col.user_long[0] = -1;
	col.user_long[1] = -1;
	col.user_long[2] = -1;
	col.user_long[3] = -1;
	col.user_long[4] = -1;
	col.user_long[5] = -1;
	col.user_short[0] = -1;
	col.user_short[1] = -1;
	col.user_short[2] = -1;
	col.user_short[3] = -1;
	col.user_short[4] = -1;
	col.user_short[5] = -1;
	col.user_short[6] = -1;
	col.user_short[7] = -1;

        if(debug){fprintf (stderr, "YEAR = %d \n",leg_year);}
	if (leg_year != 0) gmt = gmt_init (leg_year);

 	/*if (header) for (i = 0; i < 16; i++) fgets (line, 512, fp);*/	/* Skip header records */
	/* 09/08/95 jcm Header now comes in 24 lines of 80 cols OR 16 lines
		of 120 cols.  Please handle both cases */
        if(debug){fprintf (stderr, "Header flag = %d \n",header);}

 	if (header) {
 	        if(debug){fprintf (stderr, "fgets(line, 512, fp) %d \n",line);}
		fgets(line, 512, fp);

		if (line[81] == '\0')
		      {	for (i = 1; i < 24; i++) fgets (line, 512, fp);	}/* Skip header records */
		else
		      {	for (i = 1; i < 16; i++) fgets (line, 512, fp);	}/* Skip header records */
	}

        if(debug){fprintf (stderr, "Header Read %d \n",line[81]);}

        record = (struct GMT_PLUS_REC *) GMT_memory ((char *)NULL, n_alloc, sizeof (struct GMT_PLUS_REC), "mgd77togmtplus");


	rec = 0;
	while (fgets (line, 512, fp)) {
		for (i=0;i<6;i++)
			record[rec].ldata[i] = 0;
		for (i=0;i<8;i++)
			record[rec].sdata[i] = 0;
		if (!decode_record (line, t_flag,
			&record[rec].ldata[col.tsec],
			&record[rec].ldata[col.lon],
			&record[rec].ldata[col.lat],
			&record[rec].ldata[col.graw],
			&record[rec].sdata[col.gfaa-6],
			&record[rec].sdata[col.mtot-6],
			&record[rec].sdata[col.manm-6],
			&record[rec].sdata[col.twtt-6],
            		&record[rec].sdata[col.tcor-6]
            	   )) rec++;
		if (rec == n_alloc) {
			n_alloc += GMT_CHUNK;
			record = (struct GMT_PLUS_REC *) GMT_memory ((char *)record, n_alloc, sizeof (struct GMT_PLUS_REC), "mgd77togmtplus");
		}
	}
	if (fp != stdin) fclose (fp);
	free ((char *)gmt);
	n_records = rec;

	sprintf (file, "%s.gmth\0", legfname);
	if ((hpo = fopen (file, "w")) == NULL) {
		fprintf (stderr, "Could not create header file %s\n", file);
		exit (EXIT_FAILURE);
	}
	sprintf (file, "%s.gmtd\0", legfname);
	if ((fpo = fopen (file, "w")) == NULL) {
		fprintf (stderr, "Could not create data file %s\n", file);
		exit (EXIT_FAILURE);
	}

    /* assume no corrected data; fix if one was actually set */
    column = col.tcor;
    col.tcor = -1;
    for (rec=0; rec < n_records; rec++) {
       if (record[rec].sdata[column-6] != NODATA) {
          col.tcor = column;
          break;
       }
    }

    /* check for two-way data */
    column = col.twtt;
    col.twtt = -1;
    for (rec=0; rec < n_records; rec++) {
       if (record[rec].sdata[column-6] != NODATA) {
          col.twtt = column;
          break;
       }
    }

    /* check for mag total */
    column = col.mtot;
    col.mtot = -1;
    for (rec=0; rec < n_records; rec++) {
       if (record[rec].sdata[column-6] != NODATA) {
          col.mtot = column;
          break;
       }
    }

    /* check for mag anomaly */
    column = col.manm;
    col.manm = -1;
    for (rec=0; rec < n_records; rec++) {
       if (record[rec].sdata[column-6] != NODATA) {
          col.manm = column;
          break;
       }
    }

    /* check for raw gravity */
    column = col.graw;
    col.graw = -1;
    for (rec=0; rec < n_records; rec++) {
       if (record[rec].ldata[column] != NODATA) {
          col.graw = column;
          break;
       }
    }

    /* check for free-air anomaly */
    column = col.gfaa;
    col.gfaa = -1;
    for (rec=0; rec < n_records; rec++) {
       if (record[rec].sdata[column-6] != NODATA) {
          col.gfaa = column;
          break;
       }
    }

	write_gmt_header(hpo, &col, leg_year, n_records, agency, legfname, file);
	write_first_gmt_datarec(fpo, &col);

	for (rec = 0; rec < n_records; rec++) {
		if (fwrite((char *)(&record[rec]), 40, 1, fpo) != 1) {
			fprintf(stderr,"mgd77togmtplus: Error writing data record no %d\n",rec);
			exit (EXIT_FAILURE);
		}
                /****************************************************************/
                /***************************************************************
		else  fprintf (stderr,"%d ",&record[rec].ldata[col.tsec]\n);
		/****************************************************************/
		/****************************************************************/

	} fprintf (stderr, "\n");

	fclose(fpo);
	fclose(hpo);

	free ((char *)record);

	exit(0);
}

